package de.hdm_stuttgart.sd1;

import java.text.NumberFormat;
import java.time.LocalDate;
import java.time.Month;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.Locale;

public class PrintValuesByLocale {

  public static void main(String[] args) {
    final NumberFormat
      standard = NumberFormat.getInstance(),
      german = NumberFormat.getInstance(Locale.GERMANY);

    for (final Locale locale : new Locale[]{Locale.GERMANY, Locale.JAPAN, Locale.US}) {
      System.out.println("Locale " + locale.getDisplayCountry());
      print(1234567, locale);
      print(-123.456789, locale);
      print(LocalDate.of(2024, Month.DECEMBER, 22), locale);
      System.out.println("----------------------\n");
    }
  }

  /**
   * Print an int value using the given locale. Example:
   *
   * Using Locale.GERMANY a value of 1234567 should appear as "1.234.567"
   *
   * @param value The value to be formatted.
   * @param locale The locale to be used.
   */
  private static void print(final int value,  final Locale locale) {
    final NumberFormat numberFormat = NumberFormat.getInstance(locale);
    System.out.println(numberFormat.format(value));
  }

  /**
   * Print a double value using the given locale. Example:
   *
   * Using Locale.GERMANY a value of -123.456789 should appear as "-123,457"
   *
   * @param value The value to be formatted.
   * @param locale The locale to be used.
   */
  private static void print(final double value,  final Locale locale) {
    final NumberFormat numberFormat = NumberFormat.getInstance(locale);
    System.out.println(numberFormat.format(value));
  }
  /**
   * Print a {@link LocalDate} value using the given locale. Example:
   *
   * Using Locale.GERMANY the 22-nd of december 2024 should appear as "22. Dezember 2024".
   *
   * @param date The value to be formatted.
   * @param locale The locale to be used.
   */
  private static void print(final LocalDate date,  final Locale locale) {
    final DateTimeFormatter dateFormat =
      DateTimeFormatter.ofLocalizedDate(FormatStyle.LONG).withLocale(locale);
    System.out.println(dateFormat.format(date));
  }
}