package de.hdm_stuttgart.mi.sda1.jdom.background;

import java.io.IOException;
import java.util.List;

import de.hdm_stuttgart.mi.unitmarking.jdom.DomFilter;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;


/**
 * Handling address book data.
 *
 */
public class RowHighlighting implements DomFilter {

   /**
    * Read XML file data and create corresponding HTML output.
    * 
    * @param addressbook The XML input file containing address book data.
    * @throws JDOMException Internal error.
    * @throws IOException IO related errors.
    */
   @Override
   public Document process(final Document addressbook) {

      // Create <table> skeleton:
      // <table>
      //   <tr>
      //     <th>Name</th>
      //     <th>Birthday</th>
      //   </tr>
      // </table>

      final Element table  = new Element("table"),
                    trHead = new Element("tr");
      table.addContent(trHead);
      trHead.addContent(new Element("th").addContent("Name"));
      trHead.addContent(new Element("th").addContent("Birthday"));

      // Search address book for <person> elements
      @SuppressWarnings({ "unchecked", "rawtypes" }) // Uuuuuugly!!!
      final XPathExpression<Element> xpath = (XPathExpression<Element>) (XPathExpression) XPathFactory
                               .instance().compile("/addressbook/person");
      final List<Element> persons = xpath.evaluate(addressbook);

      // Now append a <tr> for each <person> input element:
      // <table>
      // ... table's column head entries, see above
      //   <tr>
      //     <td>Scott Geoffrey</td>
      //     <td>1942-07-22</td>
      //   </tr>
      //   <tr>...</tr> ...
      // </table>
      boolean setBackground = false; // keep track of table row background state.

      for (final Element person : persons) {
         final Element tr = new Element("tr");
         if ((setBackground =! setBackground)) {
            tr.setAttribute("style", "background-color: #FFFF88;");
         } 
         table.addContent(tr);
         tr.addContent(new Element("td").addContent(person.getChild("name").getText()));
         tr.addContent(new Element("td").addContent(person.getChild("birthday").getText()));
      }
      return new Document(table);
   }
}