<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    exclude-result-prefixes="xs"
    version="2.0">
    
    <xsl:output method="xml" indent="yes"/>
    <xsl:strip-space elements="*"/>
    
    <xsl:template match="/book">
        <html>
            <head>
                <title>
                    <xsl:value-of select="title"/>
                </title>
            </head>
            <body>
                <!--Element <book> may contain an @id attribute -->
                <xsl:apply-templates select="*|@id"/>
            </body>
        </html>
    </xsl:template>
    
    <xsl:template match="book/title">
        <h1>
            <xsl:value-of select="."/>
        </h1>
    </xsl:template>
    
    <xsl:template match="chapter/title">
        <h2>
            <!-- Parent element <chapter> may contain an @id attribute -->
            <xsl:apply-templates select="text()|parent::chapter/@id"/>
        </h2>
    </xsl:template>
    
    <xsl:template match="chapter">
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="para">
        <p>
            <!-- Element <para> may contain an @id attribute-->
            <xsl:apply-templates select="*|text()|@id"/>
        </p>
    </xsl:template>
    
    
    <xsl:template match="emphasis">
        <em>
            <xsl:value-of select="."/>
        </em>
    </xsl:template>
    
    <xsl:template match="link">
        <a href='#{@linkend}'>
            <xsl:value-of select="."/>
        </a>
    </xsl:template>
    
    <xsl:template match="@id">
        <xsl:attribute name="id" select="."/>
    </xsl:template>
    
    
    <!-- Handling missing templates -->
    <xsl:template match="*">
        <p>
        <xsl:text>No template defined for element '</xsl:text>
            <xsl:value-of select="name(.)"/>
            <xsl:text>'</xsl:text>
        </p>
    </xsl:template>
    
</xsl:stylesheet>