package de.hdm_stuttgart.mi.sd1.aufgabe1;

/**
 * <p>Ein Fitnesstudio bietet folgende Tarife:</p>
 *
 * <ul>
 *   <li>Jährliche Einmalzahlung, siehe {@link #TARIFTYP_JAHRESVORAUSZAHLUNG}.</li>
 *   <li>Flextarif, siehe {@link #TARIFTYP_FLEX}.</li>
 *   <li>Einmalbesuch ohne gebuchten Tarif, siehe {@link #TARIFTYP_PRO_BESUCH}.</li>
 * </ul>
 *
 * Der Rechner bietet Methoden zur Ermittlung sowohl von Kosten als auch des günstigsten Tarifs auf Basis
 * der Anzahl von Besuchen pro Jahr.
 */
public class FitnessTarifRechner {

  /**
   * <p>Für einen Jahresbeitrag von {@link #kostenJahrestarif} € kann man das Studio beliebig oft besuchen.</p>
   */
  static public final byte TARIFTYP_JAHRESVORAUSZAHLUNG = 1;

  /**
   * <p>Pro Besuch des Studios werden {@link #kostenEinmaligFlextarif} € fällig. Zusätzlich fällt eine
   * einmalige Jahresgebühr von {@link #basiskostenFlextarif} € an.</p>
   */
  static public final byte TARIFTYP_FLEX = 2;

  /**
   * <p>Ein einmaliger Besuch ohne vorab gebuchten Tarif kostet {@link #kostenEimaligOhneTarif} €.</p>
   */
  static public final byte TARIFTYP_PRO_BESUCH = 3;

  /**
   * Kosten eines Ganzjahrestickets.
   */
  static public final int kostenJahrestarif = 800;

  /**
   * Jährliche Grundkosten Flextarif.
   */
  static public final int basiskostenFlextarif = 150;

  /**
   * Kosten pro Besuch im Flextarif.
   */
  static public final int kostenEinmaligFlextarif = 5;

  /**
   * Kosten pro Besuch ohne jeglichen Tarif.
   */
  static public final int kostenEimaligOhneTarif = 12;


  /**
   * Kosten pro Jahr bei vorheriger Buchung des Flextarifs {@link #TARIFTYP_FLEX}.
   *
   * @param anzahlBesucheProJahr Anzahl der jährlichen Besuche im Studio. Negative Werte werden als
   *                             0 Besuche gewertet.
   * @return Die Jahresgesamtkosten für alle Besuche im Jahr.
   */
  static public int getKostenFlextarif(final int anzahlBesucheProJahr) {
    if (anzahlBesucheProJahr < 0) {
      return basiskostenFlextarif;
    } else {
      return basiskostenFlextarif + anzahlBesucheProJahr * kostenEinmaligFlextarif;
    }
  }

  /**
   * Kosten pro Jahr ohne Buchung eines Tarifs.
   *
   * @param anzahlBesucheProJahr Anzahl der jährlichen Besuche im Studio. Negative Werte werden als
   *                             0 Besuche gewertet.
   * @return Die Jahresgesamtkosten für alle Besuche im Jahr.
   */
  static public int getKostenOhneTarif(final short anzahlBesucheProJahr) {
    if (anzahlBesucheProJahr < 0) {
      return 0;
    } else {
      return kostenEimaligOhneTarif * anzahlBesucheProJahr;
    }
  }
  /**
   * <p>Optimaler Tariftyp basierend auf der Anzahl von Besuchsterminen pro Jahr.</p>
   *
   * <p>Beispiel: Ein Nutzer besucht das Studio an 200 Tagen im Jahr. Dies ergibt folgende
   * Gesamtkosten in den drei Tariftypen:</p>
   *
   * <dl>
   *   <dt>{@link #TARIFTYP_JAHRESVORAUSZAHLUNG}</dt>
   *   <dd><code>{@link #kostenJahrestarif}</code></dd>
   *
   *   <dt>{@link #TARIFTYP_FLEX}</dt>
   *   <dd>{@link #basiskostenFlextarif} + 200 * {@link #kostenEinmaligFlextarif}</dd>
   *
   *   <dt>{@link #TARIFTYP_PRO_BESUCH}</dt>
   *   <dd> 200 * {@link #kostenEimaligOhneTarif}</dd>
   *
   * </dl>
   *
   * @param anzahlBesucheProJahr Anzahl der jährlichen Besuche im Studio.
   *
   * @return <p>Der günstigste Tariftyp entsprechend der Anzahl an Besuchen. Falls sich für verschiedene Tariftypen
   * identische Kosten ergeben, so gilt folgende Prioritätenfolge:</p>
   * <ol>
   *   <li>{@link #TARIFTYP_PRO_BESUCH}</li>
   *   <li>{@link #TARIFTYP_FLEX}</li>
   *   <li>{@link #TARIFTYP_JAHRESVORAUSZAHLUNG}</li>
   * </ol>
   *
   */

  static public byte getOptimalenTariftyp(final short anzahlBesucheProJahr) {

    final int flextarifGesamtkosten = getKostenFlextarif(anzahlBesucheProJahr);
    if (getKostenOhneTarif(anzahlBesucheProJahr) <= flextarifGesamtkosten) {
      return TARIFTYP_PRO_BESUCH;
    } else {
      if (flextarifGesamtkosten <= kostenJahrestarif) {
        return TARIFTYP_FLEX;
      } else {
        return TARIFTYP_JAHRESVORAUSZAHLUNG;
      }
    }
  }
}