package de.hdm_stuttgart.mi.sd1.aufgabe2;

import de.hdm_stuttgart.mi.sd1.aufgabe1.Maexchen;

/**
 * <p>Darstellung eines Mäxchen Wurfs, z.B. (3|2), siehe auch {@link Maexchen}.</p>
 */
public class MaexleWurf {

  final int untereAugenzahl, obereAugenzahl;

  public MaexleWurf(final int wuerfel1, final int wuerfel2) {
    if (wuerfel1 <= wuerfel2) {
      untereAugenzahl = wuerfel1;
      obereAugenzahl = wuerfel2;
    } else {
      untereAugenzahl = wuerfel2;
      obereAugenzahl = wuerfel1;
    }
  }

  /**
   * <p>Vergleiche den aktuellen Wurf mit einem anderen Wurf: Welcher Wurf ist höher?</p>
   *
   * <table border="1" style="border-collapse: collapse;">
   *   <tr>
   *     <th colspan="14">Normalwerte</th>
   *     <th colspan="6">Pasch</th>
   *     <th>Mäxle</th>
   *   </tr>
   *   <tr>
   *     <td>(3|1)</td><td>(3|2)</td>
   *     <td>(4|1)</td><td>(4|2)</td><td>(4|3)</td>
   *     <td>(5|1)</td><td>(5|2)</td><td>(5|3)</td><td>(5|4)</td>
   *     <td>(6|1)</td><td>(6|2)</td><td>(6|3)</td><td>(6|4)</td><td>(6|5)</td>
   *
   *     <td>(1|1)</td>
   *     <td>(2|2)</td>
   *     <td>(3|3)</td>
   *     <td>(4|4)</td>
   *     <td>(5|5)</td>
   *     <td>(6|6)</td>
   *
   *     <td>(1|2)</td>
   *   </tr>
   *   <tr>
   *     <td>Niedrigster Wert</td>
   *     <td colspan="19"></td>
   *     <td>Höchster Wert</td>
   *   </tr>
   *   <caption>Reihenfolge gewürfelter Augenpaare in aufsteigender Folge im Mäxchen Spiel.</caption>
   *
   * </table>
   *
   * <p>Beispiele:</p>
   *
   * <ul>
   *   <li><code>(2| 5)</code> gleich <code>(5| 2)</code> wird »52« gesprochen und ist größer als (4| 3), gesprochen
   *   »43«</li>
   *
   *   <li><code>(2| 2)</code> ist als höher als alle nicht-Pasch / nicht-Mäxchen Werte wie z.B. <code>(5| 6)</code>,
   *   aber niedriger als das Mäxchen <code>(1|2)</code>.</li>
   *
   * </ul>
   *
   * @param wurf Ein im Spiel geworfener Wert bestehend aus den Augenzahlen zweier Würfel.
   * @return <ul>
   *   <li>positives Ergebnis, falls fremder Wert höher liegt.</li>
   *   <li>0, falls Wert gleich ist</li>
   *   <li>negatives Ergebnis, falls fremder Wert niedriger liegt.</li>
   * </ul>
   */
  public int compareTo(final MaexleWurf wurf) {
    return wurf.getWert() - getWert();
  }

  private int getWert() {
    if (Maexchen.istKeinSonderfall(untereAugenzahl, obereAugenzahl)) {
      return untereAugenzahl + 10 * obereAugenzahl; // Nix besonderes, nur zählen, maximal 65.
    } else if (Maexchen.istPasch(untereAugenzahl, obereAugenzahl)) {
      return 65 + untereAugenzahl; // Pasch: Höchster »Normalwert« + Paschwert (1-6), maximal 71.
    } else {
      return 72; // Mäxchen: Höchster Wert
    }
  }

  /**
   * Ausgabe der Augenzahlen eines Wurfs in Normaldarstellung analog den gesprochenen Werten im Spiel.
   * also
   * z.B. (5|2) statt (2|5)
   * @return
   */
  @Override
  public String toString() {
    return "(" + obereAugenzahl +  "|" + untereAugenzahl +")";
  }
}
