package de.hdm_stuttgart.mi.sda1.scripts;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class DbHandler {

	final Connection conn;
	
	final PreparedStatement insertLanguage, insertScript, insertLanguageScript;
	
	/**
	 * Opening a database connection by using fixed properties
	 * JDBC URL "jdbc:mysql://localhost:3306/hdm", user  "hdmuser" and
	 * password "XYZ"
	 * 
	 * @throws SQLException
	 */
	public DbHandler() throws SQLException {
		conn = DriverManager.getConnection("jdbc:mysql://localhost:3306/hdm", "hdmuser", "XYZ");

		insertLanguage = conn.prepareStatement("INSERT INTO Language (language) VALUES(?)", Statement.RETURN_GENERATED_KEYS);
		insertScript = conn.prepareStatement("INSERT INTO Script (xmlId, description, sourcecode) VALUES(?, ?, ?)", Statement.RETURN_GENERATED_KEYS);
	    insertLanguageScript = conn.prepareStatement("INSERT INTO LanguageScript (language, script) VALUES(?, ?)", Statement.RETURN_GENERATED_KEYS);
	}
	
	public long addLanguage(final String language) throws SQLException {
		insertLanguage.setString(1, language);
		
		if (1 == insertLanguage.executeUpdate()) {
			final ResultSet genKeys = insertLanguage.getGeneratedKeys();
			if (genKeys.next()) {
				return genKeys.getLong(1);
			}
		}
		return -1;
	}

	public void close() throws SQLException {
		conn.close();
	}
	
	public void addScript(
			final String xmlId,
			final String description,
			final String sourcecode,
			final Long[] languageIds) throws SQLException {
		insertScript.setString(1, xmlId);
		insertScript.setString(2, description);
		insertScript.setString(3, sourcecode);
		
		if (1 == insertScript.executeUpdate()) {
			final ResultSet genKeys = insertScript.getGeneratedKeys();
			if (genKeys.next()) {
				final Long scriptId = genKeys.getLong(1);
				for (final Long languageId: languageIds) {
					insertLanguageScript.setLong(1, languageId);
					insertLanguageScript.setLong(2, scriptId);
					insertLanguageScript.executeUpdate();
				}
			}
		}
		
	}
	
}
