package de.hdm_stuttgart.mi.sda1.exam.rdbms;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


/**
 * Handling database connections
 *
 */
public class DbHandler {
   private static Logger log = LogManager.getLogger(DbHandler.class);

   /**
    * Value indicating a given object does not exist.
    */
   static public final int MISSING_ID = -1;
   Connection conn = null;

   PreparedStatement insertLanguage, selectLanguageId,
                     insertNoun, selectNounId,
                     insertTranslation;

   /**
    * Connecting to our database server
    * @throws SQLException Connection failure.
    */
   public void connect() throws SQLException {
      conn = DriverManager.getConnection("jdbc:mysql://localhost:3306/hdm","hdmuser","XYZ");
      insertLanguage = conn.prepareStatement("INSERT INTO Language (lang) VALUES (?)", Statement.RETURN_GENERATED_KEYS);
      selectLanguageId = conn.prepareStatement("SELECT id FROM Language WHERE (lang=?)");
      insertNoun = conn.prepareStatement("INSERT INTO Noun (word, lang) VALUES (?, ?)", Statement.RETURN_GENERATED_KEYS);
      selectNounId = conn.prepareStatement("SELECT id FROM Noun WHERE word = ? AND lang = ?");
      insertTranslation = conn.prepareStatement("INSERT INTO Translation VALUES(?, ?)");
   }

   /**
    * @throws SQLException Unable to disconnect.
    */
   public void disconnect() throws SQLException {
      conn.close();
      conn = null;
   }

   /**
    * Connection status
    * @return true if connected, false otherwise.
    */
   public boolean isConnected() {
      return null != conn;
   }

   private int getLanguageId(final String lang) throws SQLException {
      selectLanguageId.setString(1, lang);    
      selectLanguageId.executeQuery();
      return getDatabaseId(selectLanguageId.getResultSet());
   }

   private int getNounId(final String word, final int languageId) throws SQLException {
      selectNounId.setString(1, word);
      selectNounId.setInt(2, languageId);
      selectNounId.executeQuery();
      return getDatabaseId(selectNounId.getResultSet());
   }

   private static int getDatabaseId(final ResultSet key) throws SQLException {
      if (key.next()) {
         return key.getInt(1);
      } 
      return MISSING_ID;
   }

   /**
    * Add a new language like e.g. "de"
    * 
    * @param language The language two character identifier like "en", "de", "fr", ...
    * @return The generated primary key id value.
    * @throws SQLException Language may possibly exist.
    */
   public int addLanguageIfMissing(final String language) throws SQLException {

      final int languageId = getLanguageId(language);
      if (MISSING_ID == languageId) {
         insertLanguage.setString(1, language);
         insertLanguage.executeUpdate();
         return getDatabaseId(insertLanguage.getGeneratedKeys());
      } else {
         return languageId;
      }
   }

   /**
    * Add a new noun for a given language like e.g. "winter" and language id = 12
    * 
    * @param word The word to be inserted
    * @param languageId The language to be considered.
    * @return The generated primary key id value.
    * @throws SQLException Language may possibly exist.
    */
   public int addNounIfMissing(final String word, int languageId) throws SQLException {

      final int nounId = getNounId(word, languageId);
      if (MISSING_ID == nounId) {
         insertNoun.setString(1, word);
         insertNoun.setInt(2, languageId);
         insertNoun.executeUpdate();
         return getDatabaseId(insertNoun.getGeneratedKeys());
      } else {
         return nounId;
      }
   }

   /**
    * Combining two nouns into a translation.
    * @param fromId The 'from' word and language
    * @param toId The 'to' word and language
    * 
    */
   public void addTranslation(final int fromId, final int toId) {
      try {
         insertTranslation.setInt(1, fromId);
         insertTranslation.setInt(2, toId);
         insertTranslation.executeUpdate();
      } catch (SQLException e) {
         log.info("Translation (" + fromId + ", " + toId + ") already registered: " +
               e.getMessage());
      }
   }
}