package de.hdm_stuttgart.mi.sd1.fraction;

import org.junit.jupiter.api.Test;


import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.junit.jupiter.api.Assertions.assertFalse;

import java.util.Arrays;

import de.hdm_stuttgart.mi.sd1.store.Arraymethods;


@SuppressWarnings("javadoc")
public class ArrayMethodTest {

  @Test
  public void testFindIndex(){
    assertEquals(Arraymethods.findIndex(new int[]{1, 3, -3, 5}, -3), 2);
    assertEquals(Arraymethods.findIndex(new int[]{1, -3, -3, 5}, -3), 1);
    assertEquals(Arraymethods.findIndex(new int[]{1, -3, 9, -3, 5}, -3), 1);
    assertTrue(Arraymethods.findIndex(new int[]{1, 3, 2, 5}, 4) < 0);
  }

  @Test
  public void testIsEqual() {
    assertTrue(Arraymethods.isEqual( // empty array
        new int[]{}, new int[]{}));

    assertTrue(Arraymethods.isEqual( // Just one value
        new int[]{7}, new int[]{7}));

    assertTrue(Arraymethods.isEqual(
        new int[]{1, 3, -3, 5}, new int[]{1, 3, -3, 5}));

    assertFalse(Arraymethods.isEqual(
        new int[]{1, 3, -3, 5}, new int[]{1, 3, -3}));

    assertFalse(Arraymethods.isEqual(
        new int[]{1, 3, -3}, new int[]{1, 3, -3, 5}));
  }

  @Test
  public void testSwap() {
    final int[]
        a = {1, 3, -4, 7, 7},
        b = {-5, 2, 4, 1, 12},
        aCopy = Arrays.copyOf(a, a.length),
        bCopy = Arrays.copyOf(b, b.length);

    Arraymethods.swap(aCopy, bCopy); // Swapping a and b
    assertArrayEquals(b, aCopy);
    assertArrayEquals(a, bCopy);

    Arraymethods.swap(aCopy, bCopy);// Swapping a and b again==> revert to start
    assertArrayEquals(a, aCopy);
    assertArrayEquals(b, bCopy);

    final int[] c = {-5, 2, 4};

    Arraymethods.swap(aCopy, c); // Different sizes
    assertArrayEquals(new int[]{-5, 2, 4, 7, 7}, aCopy);
    assertArrayEquals(new int[]{1, 3, -4}, c);
  }

  @Test
  public void testConcat() {
    assertArrayEquals(new int[]{}, 
        Arraymethods.concat(new int[]{}, new int[]{}));

    assertArrayEquals(new int[]{1, 8}, 
        Arraymethods.concat(new int[]{1}, new int[]{8}));
    
    assertArrayEquals(new int[]{1, 7}, 
        Arraymethods.concat(new int[]{}, new int[]{1, 7}));
    
    assertArrayEquals(new int[]{1, 7}, 
        Arraymethods.concat(new int[]{1, 7}, new int[]{}));
    
    assertArrayEquals(new int[]{1, 7, 3, -1, -2}, 
        Arraymethods.concat(new int[]{1, 7}, new int[]{3, -1, -2}));
    
    assertArrayEquals(new int[]{9, 9, 8, 8}, 
        Arraymethods.concat(new int[]{9}, new int[]{9, 8, 8}));
  }

  @Test
  public void testIsPalindrome() {
    // Examples from http://www.palindromelist.net
    
    final String[] palindromes = {
        "A Santa at Nasa", 
        "Able was I, ere I saw Elba", 
        "Borrow or rob?"};
    
    for (final String p: palindromes) {
      assertTrue(Arraymethods.isPalindrome(p), p + " is a palindrome");
    }

    final String[] nonPalindromes = {"Sea", "Test it!"};
    for (final String p: nonPalindromes) {
      assertFalse(Arraymethods.isPalindrome(p), p + " is no palindrome");
    }
  }

  @Test
  public void testContainsSameElements() {
    assertTrue(Arraymethods.containsSameElements(
        new int[]{}, new int[]{}));
    
    assertTrue(Arraymethods.containsSameElements(
        new int[]{1}, new int[]{1}));
    
    assertTrue(Arraymethods.containsSameElements(
        new int[]{1, 2}, new int[]{1, 2}));
    
    assertTrue(Arraymethods.containsSameElements(
        new int[]{1, 2}, new int[]{2, 1}));
    
    assertTrue(Arraymethods.containsSameElements(
        new int[]{1, 2, 3}, new int[]{3, 1, 2}));
    
    assertTrue(Arraymethods.containsSameElements(
        new int[]{1, 2 , 3, 1, 3}, new int[]{3, 2, 1, 1, 3}));
    
    assertFalse(Arraymethods.containsSameElements(
        new int[]{2, 2 , 3, 1, 3}, new int[]{3, 2, 1, 1, 3}));
    
  }
}
