package de.hdm_stuttgart.mi.sd1.store;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertArrayEquals;

import org.junit.jupiter.api.Test;

@SuppressWarnings("javadoc")
public class TestSorting {

   @Test
   public void empty() {

     final IntegerStore store = new IntegerStore();
     assertEquals(0, store.getNumValues());
   }

   @Test
   public void singleValue() {

     final IntegerStore store = new IntegerStore();
     final int value = 7;
     store.addValue(value);
     assertEquals(1, store.getNumValues());
     assertEquals(value, store.getValue(0));
   }

   @Test
   public void twoUnOrderedValues() {

      final int[]
            unsortedValues = {3, -1},
              sortedValues = {-1, 3};

        assertEquals(sortedValues.length, unsortedValues.length);

        final IntegerStore store = new IntegerStore();
        for (final int i: unsortedValues) {
          store.addValue(i);
        }
        assertArrayEquals(sortedValues, store.getValues());
   }

   @Test
   public void twoOrderedValues() {

      final int[]
            unsortedValues = {-1, 3},
              sortedValues = unsortedValues;

        final IntegerStore store = new IntegerStore();
        for (final int i: unsortedValues) {
          store.addValue(i);
        }
        assertArrayEquals(sortedValues, store.getValues());
   }

   
   @Test
  public void simpleSort() {
    final int[]
        unsortedValues = {2, -1, 5, 2, 7, 6},
          sortedValues = {-1, 2, 2, 5, 6, 7};

    assertEquals(sortedValues.length, unsortedValues.length);

    final IntegerStore store = new IntegerStore();
    for (final int i: unsortedValues) {
      store.addValue(i);
    }
    assertArrayEquals(sortedValues, store.getValues());
  }

  @Test
  public void duplicateSort() {
    final int[]
        unsortedValues = {0, -7, 3, 2, 3, -4, 3, 4, 5, -7},
         sortedValues  = {-7, -7, -4, 0, 2, 3, 3, 3, 4, 5};
    
    assertEquals(sortedValues.length, unsortedValues.length);

    final IntegerStore store = new IntegerStore();
    for (final int i: unsortedValues) {
      store.addValue(i);
    }
    assertArrayEquals(sortedValues, store.getValues());
  }
}
