package de.hdm_stuttgart.mi.sd1.binomial;

/**
 * Calculating <a href="https://en.wikipedia.org/wiki/Binomial">binomials</a>.
 */
public class Binomial {

  public final static int ILLEGAL_ARGUMENTS = -1;

  /**
   * Calculating binomials. Example:
   *
   *   5         5!                5*4*3*2*1
   *  ( ) = ------------ =   --------------------- = 10
   *   2     2! (5 - 2)!     3 * 2 * 1   *   2 * 1
   *
   *   Preconditons: 0 &lt;= k and k &lt;= n
   *
   * @param n
   * @param k
   * @return The binomial's value or {@link #ILLEGAL_ARGUMENTS} if preconditions are not fulfilled.
   */

  public static long binomial(int n, int k) {

    // Handle preconditions / illegal arguments
    if (k < 0 || n < k ) {
      return ILLEGAL_ARGUMENTS;
    }
    
    // Trying to avoid arithmetic overflow by making use of:
    //             n        n
    //           (   ) =  (   )
    //             k       n-k
    //
    if (n - k < k) {
      k = n - k;
    }
    
    // Numerator product (n - k + 1) ... (n - 1) n
    long result = 1;
    for (int i = n - k + 1; i <= n; i++) {
      result *= i;
    }
    for (int i = 2; i <= k; i++) {
      result /= i;
    }
    return result;
  }
}
