package de.hdm_stuttgart.mi.sda1.console_sum;

import java.util.Scanner;

/**
 * Adding user supplied values 
 * 
 */
public class App {

   /**
    * @param args Unused
    */

   public static void main(String[] args) {

      try (final Scanner scanner = new Scanner(System.in)) { // You will learn about this later (avoids scanner.close()).

         System.out.print("How many values do you want to summ up? ");
         final int numValues = scanner.nextInt();

         // Allocate an array to store the desired number of integer values
         final int[] inputValues = new int[numValues];
         int sum = 0;
         for (int i = 0; i < numValues; i++) {
            System.out.print("Enter value #" + (i + 1) + " of " + numValues + ": ");
            inputValues[i] = scanner.nextInt();
            sum += inputValues[i];
         }
         System.out.println();// Add an empty line
         // Write user input to console output
         for (int i = 0; i < numValues - 1; i++) {// Omit the last element adding '+' signs if required.
            System.out.print(inputValues[i]);
            if (0 <= inputValues[i + 1]) { // Add a '+' sign if the subsequent number is positive or zero
               System.out.print("+");
            }
         }
         // Write the last element to console output
         System.out.println(inputValues[numValues - 1] + " = " + sum);
      }
   }

   /**
    * Format a sum of an array of integer values according to the following
    * rules:
    *   1. Avoiding zeroes
    *   2. Trying to start a sum with a non-negative value
    *   3. Representing single values without equal sign by exchanging starting value with
    *      first positive value.
    *   
    *   Examples:
    *   {1,7,-4} --&gt; "1+7-4=4"
    *   {-1,7,3} --&gt; "7-1+3=9"
    *   {5} --&gt; "4"
    *   {} --&gt; "0"
    *   {0,0,0} --&gt; "0"
    *   {1,0,4,0,0,5} --&gt; "1+4+5=10"
    * 
    * @param values The set of integer values to be represented as a sum
    * @return The formatted string.
    */
   static public String prettifyOutput(final int[] values) {

      final int [] nonzeroValueContainer = new int[values.length];

      // Copy all values but zeroes: Container may even be empty!
      int nonzeroValueCount = 0;
      for (final int value : values) {
         if (0 != value) {
            nonzeroValueContainer[nonzeroValueCount++] = value;
         }
      }
      switch (nonzeroValueCount) {

      case 0: return "0";
      case 1: return "" + nonzeroValueContainer[0];
      }
      
      // Still here? We have at least two nonzero values.
      if (nonzeroValueContainer[0] < 0) {                               // The first (leftmost) value may be negative.
         int i;
         for (i = 1;                                                    // Strange loop: When finished variable i will either point
               i < nonzeroValueCount && nonzeroValueContainer[i] < 0;   // to the first positive value or has reached variable
               i++);                                                    // nonzeroValueCount (== is greater than last array index).
        
         if (i < nonzeroValueCount) {                                   // Positive value within array exists?
            final int tmp = nonzeroValueContainer[0];                   // swap positions
            nonzeroValueContainer[0] = nonzeroValueContainer[i];
            nonzeroValueContainer[i] = tmp;
         } 
      }
      int sum = nonzeroValueContainer[0];  // Initialize sum by first array value.
      final StringBuffer equationFormula = new StringBuffer("" + sum);
      for (int i = 1; i < nonzeroValueCount; i++) {
         final int value = nonzeroValueContainer[i];
         sum += value;
         if (0 <= value) { // Positive (unsigned) values need an extra '+' sign.
            equationFormula.append('+');
         }
         equationFormula.append(value);
      }
      equationFormula.append("=" + sum);
      return equationFormula.toString();
   }
}
