package de.hdm_stuttgart.mi.sd1.htmlformat;

import java.io.PrintStream;

/**
 * Formatting {@link Address} records. The generated
 * output layout can be described as:
 * 
 *   ******************************
 *   * head                       *
 *   ******************************
 *   *                            *
 *   *  body                      *
 *   *                            *
 *   ******************************
 *   * tail                       *
 *   ******************************
 */
public interface AddressFormatter {

  /**
   * When formatting an address list this method
   * will be called exactly once initiating
   * output to the head section.
   * 
   * @param out The output stream
   */
  public void printHead (PrintStream out);
  
  /**
   * This method will be called for each address record.
   * Corresponding data will populate the body section.
   * 
   * @param address The current address record.
   * @param out The output stream.
   */
  public void printRecord(Address address, PrintStream out);
  /**
   * When formatting an address list this method
   * will be called at last to finish
   * output generation of the tail section.
   * 
   * @param out The output stream
   */
  public void printTail (PrintStream out);
}