package de.hdm_stuttgart.mi.sd1.grep;

import org.junit.Assert;
import org.junit.Test;

/**
 * Testing command line handling
 */
public class CommandLineTest {
    /**
     * Testing empty command line
    * @throws ParseException 
     */
    @Test(expected=CommandLineException.class)
    public void testAppEmptyCmd() throws CommandLineException{
    	
    	// Empty command line
    	// Empty command line
    	new Grep(new String[]{});
    	// Should not reach subsequent line due to expected exception.
    	Assert.fail();
    }

    /**
     * Testing invalid command line option
     */
    @Test (expected=CommandLineException.class)
    public void testInvalidOption() throws CommandLineException {
      
      try {
         new Grep(new String[]{"-k" /* invalid option */, "dummySearchString", "dummyFileName"});
         Assert.fail();
      } catch (final CommandLineException e) {
         Assert.assertEquals("Invalid option '-k'", e.getMessage());
         throw e;
      }
    }

    /**
     * Testing single file
     */
    @Test
    public void testAppSingleFile() throws CommandLineException {
      
      final String testfileName = "my.txt";
      
      // Empty command line
      final Grep grep = new Grep(new String[]{"dummySearchString", testfileName});
      
      // Checking for defaults
      Assert.assertTrue(grep.showMatchingData);
      
      Assert.assertNotNull(grep.filenames);
      Assert.assertEquals(1, grep.filenames.length);
      Assert.assertEquals(testfileName, grep.filenames[0]);
    }

    /**
     * Testing single file
    * @throws ParseException 
     */
    @Test
    public void testAppMultipleFiles() throws CommandLineException {
    	
    	final String[] cmdLineArgs = new String[] {"dummySearchString", "my.txt", "b.txt", "c.txt"};
    	
    	// Empty command line
    	final Grep grep = new Grep(cmdLineArgs);
    	
    	// Checking for defaults
    	Assert.assertTrue(grep.showMatchingData);
    	
    	Assert.assertNotNull(grep.filenames);
    	Assert.assertEquals(cmdLineArgs.length - 1, grep.filenames.length);
    	
    	
    	for (int i = 0; i < cmdLineArgs.length - 1; i++) {
        	Assert.assertEquals(cmdLineArgs[i + 1], grep.filenames[i]);
    	}    	
    }

    /**
     * Testing single file ignoring case
    * @throws ParseException 
     */
    @Test
    public void testAppSingleFileIgnoreCase() throws CommandLineException {
    	
    	final String testfileName = "my.txt";
    	
    	// Empty command line
    	final Grep grep = new Grep(new String[]{"-i", "dummySearchString", testfileName});
    	
    	// Checking for defaults
    	Assert.assertTrue(grep.showMatchingData);
    	
    	Assert.assertNotNull(grep.filenames);
    	Assert.assertEquals(1, grep.filenames.length);
    	Assert.assertEquals(testfileName, grep.filenames[0]);
    }
    /**
     * Testing single file hiding matches
    * @throws ParseException 
     */
    @Test
    public void testAppSingleFileHideMatch() throws CommandLineException {
    	
    	final String testfileName = "my.txt";
    	
    	// Empty command line
    	final Grep grep = new Grep(new String[]{"-l", "dummySearchString", testfileName});
    	
    	// Checking for defaults
    	Assert.assertFalse(grep.showMatchingData);
    	
    	Assert.assertNotNull(grep.filenames);
    	Assert.assertEquals(1, grep.filenames.length);
    	Assert.assertEquals(testfileName, grep.filenames[0]);
    }
}
