package de.hdm_stuttgart.mi.sd1.textstat;

import java.util.Comparator;

/**
 * A helper class to account for frequencies of words found in textual input. 
 *
 */
public class WordFrequency {
  /**
   * The frequency of this word will be counted.
   */
  public final String word;
  private int frequency;
  
  /**
   * @param word The immutable value. Has to be non-null.
   */
  public  WordFrequency(final String word) {
    this.word = word;
    frequency = 1;
  }
  
  /**
   * We found another one. Thus increment the total number
   * to be returned by {@link #getFrequency()}.
   */
  public void incrementFrequency() {
    frequency++;
  }
  /**
   * @return The number of appearances so far.
   */
  public int getFrequency() {
    return frequency;
  }

  @Override
  public int hashCode() {
    return word.hashCode();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj instanceof WordFrequency) {               // WordFrequency instances are equal if and
      return ((WordFrequency) obj).word.equals(word); // only if the underlying words are equals
    } else {
      return false;
    }
  }
  /**
   * Compare by word frequencies
   *
   */
  public static class CompareByFrequency implements Comparator<WordFrequency> {

    @Override
    public int compare(WordFrequency wf1, WordFrequency wf2) {
      final int frequencyDifference = wf2.frequency - wf1.frequency;
      if (0 == frequencyDifference) { // Frequencies are equal, thus sort alphabetically
        return wf1.word.compareTo(wf2.word);
      } else {
        return frequencyDifference;
      }
    }
  }
}
