package de.hdm_stuttgart.sd1.math;

import static org.junit.Assert.*;

import org.junit.Test;

/**
 * Testing exponential implementation
 *
 */
public class MaxAbsTest{

  /**
   * Absolute value.
   */
  @Test
  public void testAbsoluteValue() {

    assertTrue(nearlyEqualCheck(Math.abs(4.1), 4.1, 1E-14));
    assertTrue(nearlyEqualCheck(Math.abs(-3.1), 3.1, 1E-14));
    assertTrue(nearlyEqualCheck(Math.abs(0.), 0., 1E-14));
  }
  
  /**
   * Max value.
   */
  @Test
  public void testMaxValue() {
    assertTrue(nearlyEqualCheck(Math.max(4.1, 5.2), 5.2, 1E-14));
    assertTrue(nearlyEqualCheck(Math.max(-4.1, 3.2), 3.2, 1E-14));
    assertTrue(nearlyEqualCheck(Math.max(3.2, -4.1), 3.2, 1E-14));
    assertTrue(nearlyEqualCheck(Math.max(-5.7, -4.1), -4.1, 1E-14));
    assertTrue(nearlyEqualCheck(Math.max(-4.1, -5.7), -4.1, 1E-14));
  }
  
  /**
   * Test, whether a given value value is within the
   * interval [reference - epsilon, reference + epsilon]
   * 
   * This is helpful to assure two values are identical within
   * a given tolerance limit being impose by e.g. limited
   * computational precision.
   *   
   * @param value The value to be tested
   * @param reference The center of the interval
   * @param epsilon Twice the inverval's length
   * @return
   */
  static boolean nearlyEqualCheck(double value, double reference, double epsilon) {
    return reference - epsilon < value && value < reference + epsilon;
  }
}