package de.hdm_stuttgart.sd1.math;

/**
 * Computing the factorial.
 */
public class Math {
    
  /**
   * Computing the factorial of a given argument.
   * 
   * @param n
   *  Zero or any positive integer 
   * 
   * @return
   *  The product 1 x 2 x 3 x ... x n or 1 in case of n == 0.
   *  In case of an arithmetic overflow a value of {@link Long#MAX_VALUE} is being returned.
   */
  static public long factorial(final int n) {
    
    long ret = 1;
    
    for (int i = n; 1 < i; i--) {
      
      // Alternate use of exceptions see related discussion in
      //  http://stackoverflow.com/questions/1657834/how-can-i-check-if-multiplying-two-numbers-in-java-will-cause-an-overflow
      if (i < Long.MAX_VALUE / ret ) {// Expecting overflow??
        ret *= i;
      } else {
        return Long.MAX_VALUE; // Multiplication overflow error
      }
    }
    return ret;
  }
  
  /**
   * Computing the factorial of a given argument.
   * 
   * @param n
   *  Zero or any positive integer 
   * 
   * @return
   *  The product 1 x 2 x 3 x ... x n or 1 in case of n == 0.
   *  In case of an arithmetic overflow a value of {@link Long#MAX_VALUE} is being returned.
   */
  static public long factorialRecurse(final int n) {
    if (0 == n) {
        return 1;                         // Termination condition
    } else {
      return n * factorialRecurse(n - 1); // Reducing step: n! = n * (n - 1)!
    }
  }
}