package de.hdm_stuttgart.sd1.math;

/**
 * <p>This class implements a subset of functions from class
 * {@link java.lang.Math} using power series expansions.</p>
 *
 */
public class Math {
  
  static int seriesLimit = 5;
  
  /**
   *  
   * @param seriesLimit The last term's index of a power series to be included.
   *   
   */
  public static void setSeriesLimit(int seriesLimit) {
    Math.seriesLimit = seriesLimit;
  }
  
  /**
   * <p>Approximating the natural exponential function by a finite
   * number of terms using power series expansion.</p>
   *
   * \[ \begin{aligned}
   *   e^x ={} &amp; 1 + {x\over 1!} + {x^2\over 2!} + {x^3\over 3!} + \dots \\
   *       ={} &amp; \sum_{i = 0}^\infty {x^i\over i!}
   * \end{aligned} \]
   *
   * A power series implementation has to be finite since an 
   * infinite number of terms requires infinite execution time.
   * 
   * The number of terms to be considered can be set by {@link #setSeriesLimit(int)}}
   * 
   * @param x The exponential's argument as in  \( e^x \)

   * @return The value \( e^x \) itself.
   */
  public static double exp(double x) {
    double currentTerm = 1.,  // the first (i == 0) term x^0/0!
        sum = currentTerm;    // initialize to the power series' first term
    
    for (int i = 1; i <= seriesLimit; i++) { // i = 0 has already been completed.
      currentTerm *= x / i;
      sum += currentTerm;
    }
    return sum;
  }
}