package de.hdm_stuttgart.mi.sda1.alignimg.handler;

import java.util.List;
import java.util.Vector;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

public class SearchAlignContentHandler implements ContentHandler {
	private static final Logger log = LogManager.getLogger(SearchAlignContentHandler.class);
	
	public final List<Img> alignImages = new Vector<Img>();

	Locator locator = null;
	
	@Override
	public void setDocumentLocator(Locator locator) {
		this.locator = locator;
	}

	@Override
	public void startDocument() throws SAXException {
		log.info(displayWithLocation("Opening document"));
		alignImages.clear();
	}

	@Override
	public void endDocument() throws SAXException {
		log.info(displayWithLocation("Closing document"));
	}

	@Override
	public void startPrefixMapping(String prefix, String uri)
			throws SAXException {
		log.info(displayWithLocation("Start mapping prefix '" + prefix + "' to '" + uri + "'"));
	}

	@Override
	public void endPrefixMapping(String prefix) throws SAXException {
		log.info(displayWithLocation("End mapping prefix '" + prefix + "'"));
	}

	@Override
	public void startElement(final String uri, final String localName, final String qName,
			Attributes atts) throws SAXException {
	   
	   if (qName.equals("img")) {
	      final String alignValue = atts.getValue("align");
	      if (null != alignValue) {
	         alignImages.add(new Img(atts.getValue("src"), alignValue));
	      }
	   }
	   
		log.info(displayWithLocation("Opening Element '" + localName + "' of namespace " + uri + "', full name='" + qName + "'"));
	}

	@Override
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		log.info(displayWithLocation("Closing Element '" + localName + "' of namespace " + uri + "', full name='" + qName + "'"));
	}

	@Override
	public void characters(char[] ch, int start, int length)
			throws SAXException {
		final String content =  new String(ch, start, length).trim();
		if (content.equals("")) {
			log.info(displayWithLocation("Textual content is just whitespace"));
		} else {
			log.info(displayWithLocation("Textual content '" + new String(ch, start, length) + "'"));
		}
	}

	@Override
	public void ignorableWhitespace(char[] ch, int start, int length)
			throws SAXException {
		log.info(displayWithLocation("Ignorable whitespace"));
	}

	@Override
	public void processingInstruction(String target, String data)
			throws SAXException {
		log.info(displayWithLocation("Processing instruction '" + target + "' with value '" + data + "'"));
	}

	@Override
	public void skippedEntity(String name) throws SAXException {
		log.info(displayWithLocation("Skipped entity '" + name + "'"));
	}
	
	String displayWithLocation(final String saxMsg) {
		if (null == locator) {
			return saxMsg;
		} else {
			return "File position (" + locator.getLineNumber() + ", " + locator.getColumnNumber() + "): " + saxMsg;
		}
	}
}