package de.hdm_stuttgart.mi;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import javax.servlet.annotation.WebServlet;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.mysql.jdbc.exceptions.jdbc4.MySQLIntegrityConstraintViolationException;
import com.vaadin.annotations.Theme;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.annotations.Widgetset;
import com.vaadin.data.validator.EmailValidator;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.server.ClientConnector.DetachListener;
import com.vaadin.ui.Button;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;

@Theme("mytheme")
@Widgetset("de.hdm_stuttgart.mi.MyAppWidgetset")
public class LoginUI extends UI implements DetachListener {

   private static final long serialVersionUID = 9013400739498375365L;
   
   private static Logger log = LogManager.getLogger(LoginUI.class);

   Statement stmt;
   
   private final TextField
      nameField = new TextField(), 
      emailField = new TextField();

   private Button
      insertButton = new Button("Insert");

	public LoginUI () {
	   final String jdbcUrl = "jdbc:mysql://localhost:3306/hdm?useSSL=false";
	   try {
		final Connection conn = DriverManager.getConnection(
				jdbcUrl, "hdmuser", "XYZ");
		stmt = conn.createStatement();
	   } catch (SQLException ex) {
	      log.error("Unable to connect to database using '" + jdbcUrl + "':", ex);
	      stmt = null;
	   }
	}
	
	@Override
	public void detach(DetachEvent event) {
	   if (null != stmt) {
	      try {
	         stmt.getConnection().close();
	      } catch (final SQLException ex) {
	         log.error("Unable to disconnect from database:", ex);
	      }
	      stmt = null;
	   }
	}

   void conditionallyActivateInsertButton() {
      final boolean isValid = 
            0 < nameField.getValue().trim().length() &&
            0 < emailField.getValue().trim().length() &&
            emailField.isValid();
      
      insertButton.setEnabled(isValid);
   }
	
	@Override
	protected void init(final VaadinRequest vaadinRequest) {
	   
		final GridLayout layout = new GridLayout(2, 3);
		layout.setMargin(true);
		setContent(layout);

		
      if (null == stmt) { // Database access failed? No hope ...
         layout.addComponent(new Label("Unable to connect to database server, see log for details"), 0, 0);
         return;
      }
      // Assembling GUI components
		layout.addComponent(new Label("Name:"), 0, 0);
		layout.addComponent(nameField, 1, 0);

		layout.addComponent(new Label("Email:"), 0, 1);
		layout.addComponent(emailField, 1, 1);

		layout.addComponent(insertButton, 1, 2);
		
		// Inserting requires valid user input.
		insertButton.setEnabled(false);

		// Caveat: Vaadin's validator will refuse e.g. "domainregistry@de"
		// and may generally be non-RFC compliant.
	   emailField.addValidator(new EmailValidator("Not a valid email"));
		
		// Check for global user input validity 
		nameField.addValueChangeListener(event -> conditionallyActivateInsertButton());
		emailField.addValueChangeListener(event -> conditionallyActivateInsertButton());
				
		insertButton.addClickListener(event -> {
		   final String sql = "INSERT INTO Person VALUES('" + nameField.getValue() + "', '" 
               + emailField.getValue() +  "')";
         try {
            final int updateCount = stmt.executeUpdate(sql);
            emailField.clear();
            nameField.clear();
            Notification.show("Successfully executed \n'" + sql + "'\ninserting "
                  + updateCount + " dataset");

         } catch (final MySQLIntegrityConstraintViolationException ex) {// Duplicate email?
            Notification.show("E-Mail already in database", Notification.Type.ERROR_MESSAGE);
         }	catch(final SQLException ex) {// Other SQL problems
            Notification.show("Failed to insert record, see log for details", Notification.Type.ERROR_MESSAGE);
            log.error("JDBC exception on data insert:", ex);
         }
		});
	}

	@WebServlet(urlPatterns = "/*", name = "LoginUIServlet", asyncSupported = true)
	@VaadinServletConfiguration(ui = LoginUI.class, productionMode = false)
	public static class LoginUIServlet extends VaadinServlet {
		private static final long serialVersionUID = -8657597788907698974L;
	}
}
