package de.hdm_stuttgart.sda1.insert;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * <p>A JDBC "getting started" example inserting person's data into a table</p>
 *
 * <p>Caveat: No error handling whatsoever is being supplied!</p>
 *
 * <p>Preconditions:</p>
 *
 * <ul>
 *   <li>A database matching {@link #main(String[])}'s connection setup must be configured.</li>
 *   <li>A database table »Person« corresponding to this project's file »resources/schema.sql« must exist.</li>
 * </ul>
 *
 */
public class SimpleInsert {

  /**
   * <p>Disclaimer:</p>
   *
   * <ul>
   *   <li>User credentials (name and password) are being hard-coded requiring compile time
   *   database configuration. This is absolutely uuuuuuugly!</li>
   *   <li>Furthermore no exception handling whatsoever is being supplied.</li>
   * </ul>
   *
   * <p>In other words: **Never** do this in any professional setting.</p>
   *
   * @param args unused
   * @throws SQLException possible causes:
   * <ul>
   *   <li>Driver matching JDBC URL unavailable.</li>
   *   <li>Missing network link to database server.</li>
   *   <li>Invalid credentials.</li>
   *   <li>Database table 'Person' does not exist.</li>
   *   <li>duplicate email (UNIQUE key).</li>
   *   <li>... something forgotten to think about?</li>
   * </ul>
   */
  public static void main(String[] args) throws SQLException {

    // Step 1: Open a connection to the database server requiring
    //         DBC URL, username and password.
    final Connection conn = DriverManager.getConnection (
      "jdbc:mysql://localhost:3306/hdm?serverTimezone=UTC&useSSL=true",
      "hdmuser",
      "XYZ");

    // Step 2: Create a Statement instance
    final Statement stmt = conn.createStatement();

    // Step 3: Execute the desired INSERT
    final int updateCount = stmt.executeUpdate(
      "INSERT INTO Person VALUES('Jim', 'jim@foo.org')");

    // Step 4: Provide feedback on execution. (However in case of
    // an Exception we won't get here at all, sigh!)
    System.out.println("Successfully inserted " + updateCount + " dataset(s)");
  }
}
