package de.hdm_stuttgart.sda1.insert;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.sql.*;
import java.util.ResourceBundle;
import java.util.Scanner;

/**
 * <p>A JDBC "getting started" example inserting person's data into a table</p>
 *
 * <p>Caveat: No error handling whatsoever is being supplied!</p>
 *
 * <p>Preconditions:</p>
 *
 * <ul>
 *   <li>A database matching {@link #main(String[])}'s connection setup must be configured.</li>
 *   <li>A database table »Person« corresponding to this project's file »resources/schema.sql« must exist.</li>
 * </ul>
 *
 */
public class SimpleInsert {

  static private final Logger log = LogManager.getLogger (SimpleInsert.class);

  static final ResourceBundle jdbcProperties = ResourceBundle.getBundle("jdbc");

  /**
   * <p>Disclaimer:</p>
   *
   * <ul>
   *   <li>User credentials (name and password) are being hard-coded requiring compile time
   *   database configuration. This is absolutely uuuuuuugly!</li>
   *   <li>Furthermore no exception handling whatsoever is being supplied.</li>
   * </ul>
   *
   * <p>In other words: **Never** do this in any professional setting.</p>
   *
   * @param args unused
   * @throws SQLException possible causes:
   * <ul>
   *   <li>Driver matching JDBC URL unavailable.</li>
   *   <li>Missing network link to database server.</li>
   *   <li>Invalid credentials.</li>
   *   <li>Database table 'Person' does not exist.</li>
   *   <li>duplicate email (UNIQUE key).</li>
   *   <li>... something forgotten to think about?</li>
   * </ul>
   */
  public static void main(String[] args) throws SQLException {

    final Connection conn = DriverManager.getConnection(
      jdbcProperties.getString("jdbcurl"),
      jdbcProperties.getString("username"),
      jdbcProperties.getString("password"));
      handleUserInput(conn.createStatement());
      conn.close();
  }
  static private void handleUserInput(final Statement statement) throws SQLException {
    try (final Scanner scanner = new Scanner(System.in)) {

      while(true) {
        System.out.print("Enter a person's name or 'x' to exit: ");
        final String name = scanner.nextLine().trim();
        if (name.equals("x")) {
          break;
        }
        System.out.print("Enter " + name + "'s email or 'x' to exit: ");
        final String email = scanner.nextLine().trim();
        if (email.equals("x")) {
          break;
        }
        final int insertCount = insertPerson(statement, name, email);
        if (1 == insertCount) {
          System.out.println("Successfully inserted new user '" + name + "'\n");
        } else {
          System.out.println("Insertion failed, duplicate email '" + email + "' ?\n");
        }
      }
      System.out.println("Bye!");
    }
  }
  /**
   * <p>Try inserting new Person record.</p>
   *
   * @param statement To be used for SQL INSERT attempt.
   * @param name Person's name
   * @param email Person's email
   * @return Inserted reord count: 1 on successful INSERT, 0 in case of duplicate email violating UNIQUE constraint.
   *
   * @throws SQLException To be thrown in case of non - {@link SQLIntegrityConstraintViolationException}
   * errors.
   */
  static public int insertPerson(final Statement statement, final String name, final String email)
    throws SQLException {
    final String sql = "INSERT INTO Person VALUES('" + name + "', '" + email + "')";
    log.info("Executing »" + sql + "«");

    try {
      return statement.executeUpdate(sql);
    } catch (final SQLIntegrityConstraintViolationException ex) {
      log.info("Constraint violation, possibly inserting duplicate email '" + email + "'");
      return 0;
    }
  }
}