package de.hdm_stuttgart.mi.sda1.rdbmsxml2html;
import java.sql.*;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
/**
 * Reading price related data.
 */
public class DbAccess {

   static private final Logger log = LogManager.getLogger(DbAccess.class);

   private static final String sqlPriceQuery = 
         "SELECT price FROM Product WHERE orderNo = ?";
   private PreparedStatement priceQuery = null;
   private Connection conn = null;


   /** Open a connection to a RDBMS.
    * @param jdbcUrl The database server's address
    * @param userName The user's name
    * @param password The user's password
    */
   public void connect(final String jdbcUrl, 
         final String userName, final String password) {
      try {
         conn = DriverManager.getConnection(jdbcUrl, userName, password);
         priceQuery = conn.prepareStatement(sqlPriceQuery);
      } catch (final SQLException e) {
         System.err.println("Unable to open connection to database:" + e);}
   }
   /** Read the price from the RDBMS for a given article number. In a "real"
    *  application we'd use an integer type rather than a string.
    * @param articleNumber
    *   The number of the article for which we search pricing information.
    * @return
    *   The price if available, else an error message.
    */
   public String readPrice(final String articleNumber) {

      try {
         priceQuery.setString(1, articleNumber);
      } catch (SQLException e) {
         log.fatal("Unable to set price:" + e);
         System.exit(1);
      }
      try (final ResultSet rs = priceQuery.executeQuery();){

         if (rs.next()) {
            return rs.getString("price");
         } else {
            return "No price available for article '" + articleNumber + "'";
         }
      } catch (final SQLException e) {
         log.error("Unable to read price:", e);
         return "Error reading price for article '" + articleNumber + "':" + e;
      }
   }
   /**
    * Close the database connection.
    */
   public void close() {
      try {conn.close();} catch (SQLException e) {
         System.err.println("Error closing database connection:" + e);
      }
   }
}