package de.hdm_stuttgart.mi.sda1.rdbmsxml2html;

import java.io.IOException;
import java.io.PrintStream;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.Document;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;
import org.jdom2.transform.XSLTransformException;
import org.jdom2.transform.XSLTransformer;
import org.jdom2.xpath.XPathFactory;

/**
 * Transforming an RDBMS data enriched XML document to HTML using
 * an XSL stylesheet.
 *
 */
public class XmlRdbms2Html {

   static private final Logger log = LogManager.getLogger(XmlRdbms2Html.class);

   private final SAXBuilder builder = new SAXBuilder();

   final DbAccess db = new DbAccess();

   final XSLTransformer transformer;
   Document catalog;

   final org.jdom2.xpath.XPathExpression<Object> selectProducts = 
         XPathFactory.instance().compile("/catalog/product");

   /**
    * @param xslFilename the stylesheet being used for subsequent
    * transformations by {@link #transform(String, String)}.
    * 
    * @throws XSLTransformException XSL script may be ill-defined
    */
   public XmlRdbms2Html(final String xslFilename) throws XSLTransformException {
      builder.setErrorHandler(new MySaxErrorHandler(System.err));
      transformer =  new XSLTransformer(xslFilename);
   }

   /**
    * The actual workhorse carrying out the transformation
    * and adding prices from the database table.
    * 
    * @param xmlInFilename input file to be transformed
    * @param htmlResultFilename the result file holding the generated HTML document
    */
   public void transform(final String xmlInFilename,
         final String htmlResultFilename) {

      try {
         catalog = builder.build(xmlInFilename);
         addPrices();
         serializeHtml(transformer.transform(catalog), htmlResultFilename);

      } catch (final IOException e) {
         log.fatal("Unable to read xml input catalog:", e);
      } catch (final JDOMException e) {
         log.fatal("Unable to transform xml to html:", e);
      }
   }

   static void serializeHtml(final Document htmlResult, final String htmlResultFilename) {
      // Set formatting for the XML output
      final Format outFormat = Format.getPrettyFormat();

      try (final PrintStream htmlOut = new PrintStream(htmlResultFilename)) {
         final XMLOutputter printer = new XMLOutputter(outFormat);
         printer.output(htmlResult, htmlOut);
      } catch (final IOException e) {
         log.error("Unable to open " + htmlResultFilename + " for writing:", e);
         System.exit(1);
      }
   }

   private void addPrices() {
      db.connect("jdbc:mysql://localhost:3306/hdm?useSSL=false", "hdmuser", "XYZ");
      
      catalog.getRootElement().
      getChildren("product").
      stream().
      forEach(p -> p.setAttribute("price", db.readPrice(p.getAttributeValue("id"))));
      
      db.close();
   }
}
