package de.hdm_stuttgart.mi.sda1.saxerrorhandler;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.junit.Assert;
import org.junit.Test;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import de.hdm_stuttgart.mi.sda1.saxerrorhandler.handler.SaxErrorHandler;

/**
 * Testing {@link SaxErrorHandler}
 */
public class AppTest {
    final static String xmlDocumentInstanceFilename = "src/test/testdata.xml";
	
	final XMLReader xmlReader;

	final ByteArrayOutputStream baOs = new ByteArrayOutputStream();
	final PrintStream ps = new PrintStream(baOs);
	
	/**
	 * Parser instantiation
	 * 
	 * @throws SAXException
	 * @throws ParserConfigurationException
	 * @throws IOException 
	 */
	public AppTest() throws SAXException, ParserConfigurationException, IOException {
    	final SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
    	
        final SAXParser saxParser = saxParserFactory.newSAXParser();
        xmlReader = saxParser.getXMLReader();
        
        xmlReader.setFeature("http://xml.org/sax/features/namespaces", true);
        xmlReader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
        
        xmlReader.setContentHandler(new DefaultHandler()); 
        xmlReader.setErrorHandler(new SaxErrorHandler(ps));
	}
	
    /**
     * The XML input is not well-formed and should thus yield a fatal error
     * thereby throwing a {@link SAXParseException}.
     * @throws IOException not expected to be thrown in this test case.
     * @throws SAXException 
     */
    @Test(expected=SAXException.class)
    public void testApp() throws IOException, SAXException {
        try {
			xmlReader.parse(xmlDocumentInstanceFilename);
		} catch (final SAXException e) {
			System.out.println("baos=" + baOs.toString());
			
			final String expectedParsingErrorMessage=
					"[Fatal Error] line 5, column 5:The end-tag for element type \"content:person\" must end with a '>' delimiter.";
			Assert.assertEquals(expectedParsingErrorMessage, baOs.toString());
			throw e;
		}
    }
}
