package de.hdm_stuttgart.mi.sda1.xmlstatistics;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import de.hdm_stuttgart.mi.sda1.saxerrorhandler.handler.SaxErrorHandler;
import de.hdm_stuttgart.mi.sda1.xmlstatistics.handler.SaxContentHandler;


/**
 * A simple SAX parser demo
 * 
 */
public class App {
    private static final Logger log = LogManager.getLogger(App.class);

    /**
     * @param args Unused
     * @throws SAXException 
     * @throws ParserConfigurationException 
     * @throws IOException 
     */
    public static void main( String[] args ) throws ParserConfigurationException, SAXException, IOException {
    	
    	log.info("Creating SAX parser factory");
    	final SAXParserFactory saxPf = SAXParserFactory.newInstance();
    	
    	log.info("Creating SAX parser");
        final SAXParser saxParser = saxPf.newSAXParser();
        final XMLReader xmlReader = saxParser.getXMLReader();
        
        log.info("Enabling namespaces and namespace prefix processing");
        xmlReader.setFeature("http://xml.org/sax/features/validation", false);
        xmlReader.setFeature("http://xml.org/sax/features/namespaces", true);
        xmlReader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
        
        log.info("Registering content- and error handler instances");
        final SaxContentHandler contentHandler = new SaxContentHandler();
        xmlReader.setContentHandler(contentHandler); 
        xmlReader.setErrorHandler(new SaxErrorHandler());
        
        final String xmlDocumentInstanceFilename = "sample.xsl";
        log.info("Start parsing file '" + xmlDocumentInstanceFilename + "'");
        xmlReader.parse(xmlDocumentInstanceFilename);
        contentHandler.printStatistics();
        
    }
}
