package de.hdm_stuttgart.mi.sda2.account;

import java.sql.Connection;
import java.sql.SQLException;

import javafx.application.Application;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.GridPane;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.stage.Stage;
import de.hdm_stuttgart.mi.sda2.account.gui.ExceptionDialog;
import de.hdm_stuttgart.mi.sda2.account.gui.NumberField;
import de.hdm_stuttgart.mi.sda2.account.sql.DbHandler;

public class TransferDriver extends Application {

	final DbHandler dbh = new DbHandler(new ExceptionDialog());
	
    final NumberField
		 srcAccountNoField = new NumberField()
		,amountField = new NumberField()
		,destAccountNoField = new NumberField();
    
    final Button
		commitBtn = new Button("Commit transaction"),
		abortBtn = new Button("Abort transaction");
    
    public static void main( String[] args ) {
    	launch(args);
    }
    public TransferDriver() {
    	
    	initEditor();
    	
    	amountField.textProperty().addListener(event -> {
    		destAccountNoField.setDisable(false);
    	});
    	
    	destAccountNoField.textProperty().addListener(event -> {
    		
    		if (!amountField.isDisabled()) { // We need a state representative
    			// Start transaction, values can no longer be changed
        		srcAccountNoField.setDisable(true);
        		amountField.setDisable(true);
        		
        		// Allow abort and commit transaction
        		abortBtn.setDisable(false);
        		commitBtn.setDisable(false);
        		
        		dbh.changeBy(srcAccountNoField.getValue(), - amountField.getValue());
    		}
    	});
    	
    	commitBtn.setOnAction(event-> {
    		dbh.changeBy(destAccountNoField.getValue(), amountField.getValue());
    		dbh.commit();
    		initEditor();
    		});
    	abortBtn.setOnAction(event-> {
    		dbh.rollback();
    		initEditor();
    	});
    }
    
    @Override
    public void start(final Stage primaryStage) throws SQLException {
    	
    	dbh.connect(Connection.TRANSACTION_SERIALIZABLE, false);


        primaryStage.setTitle("Account transfer!");
        
        final GridPane grid = new GridPane();
        grid.setAlignment(Pos.CENTER);
        grid.setHgap(10);
        grid.setVgap(10);
        grid.setPadding(new Insets(25, 25, 25, 25));
        
        int currentRowIndex = 0;
        final Text scenetitle = new Text("Account transfer");
        scenetitle.setFont(Font.font("Tahoma", FontWeight.NORMAL, 20));
        grid.add(scenetitle, 0, currentRowIndex, 2, 1);
        
        currentRowIndex++;
        grid.add(new Label("Source account:"), 0, currentRowIndex);
        grid.add(srcAccountNoField, 1, currentRowIndex);

        currentRowIndex++;
        grid.add(new Label("Amount:"), 0, currentRowIndex);
        grid.add(amountField, 1, currentRowIndex);
        
        currentRowIndex++;
        grid.add(new Label("Destination account:"), 0, currentRowIndex);
        grid.add(destAccountNoField, 1, currentRowIndex);

        currentRowIndex++;
        grid.add(abortBtn, 0, currentRowIndex);
        grid.add(commitBtn, 1, currentRowIndex);
        
        Scene scene = new Scene(grid, 300, 250);
        primaryStage.setScene(scene);
        primaryStage.show();
        
    }
    
    @Override
	public void stop() throws Exception {
		super.stop();
        dbh.close();
	}
	private void initEditor() {
    	// init field values
    	srcAccountNoField.setText("");
    	amountField.setText("");
    	destAccountNoField.setText("");
    	
    	// Set disabled state of fields and buttons
    	srcAccountNoField.setDisable(false);
    	amountField.setDisable(false);
    	destAccountNoField.setDisable(true);
    	
    	abortBtn.setDisable(true);
    	commitBtn.setDisable(true);
    }
}
