package de.hdm_stuttgart.mi.sda2.hjpa;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.junit.Assert;
import org.junit.Test;

import de.hdm_stuttgart.mi.sda2.hellojpa.CreateEntity;
import de.hdm_stuttgart.mi.sda2.hellojpa.model.Airline;

/**
 * Dealing with insertion and reading entities.
 */
public class EntityTest {
    /**
     * Creating a database entity and testing successful reload by primary key.
     */
    @Test
    public void testApp() {
       final EntityManagerFactory factory = Persistence.createEntityManagerFactory(CreateEntity.PERSISTENCE_UNIT_NAME);
       final EntityManager em = factory.createEntityManager();
       
       final String airlineName = "Lufthansa",
             icaoCode = "DLH";

       final Long id; // Will be initialized by database generated primary key value.
       
       em.getTransaction().begin(); // Creating and storing an entity
       {
          final Airline lh = new Airline(airlineName, icaoCode);
          
          em.persist(lh);
          id = lh.getId();
       }
       em.getTransaction().commit();
       
       em.getTransaction().begin(); // Reloading the entity by its primary key
       {
          final Airline readLh = em.find(Airline.class, id);
          Assert.assertEquals(airlineName, readLh.getName());
          Assert.assertEquals(icaoCode, readLh.getIcaoCode());
       }
       em.getTransaction().commit();
       
       em.close();
       factory.close();
    }
}
