<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  xmlns="http://docbook.org/ns/docbook" xmlns:db="http://docbook.org/ns/docbook">
  
  
  <xsl:include href="common.xsl"/>
  
  <xsl:param name="generateIdPrefix">__sdwordm4342cs__</xsl:param>
  <xsl:output indent="no"/>
  <xsl:param name="lastAnswerId" select="document('lastAnswerId.xml',/)/lastAnswer/@id" />
  
  <!-- Excluding glossary self references  -->
  <xsl:key name="internRef" match="//*[@linkend and not(ancestor::db:glossary)]" use="@linkend"/>
  
  <xsl:template match="node() | @*">
    <xsl:copy>
      <xsl:apply-templates select="node()| @*"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="*" mode="hdmIdGenerate">
    <xsl:choose>
      <xsl:when test="@xml:id">
        <xsl:value-of select="@xml:id"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$generateIdPrefix"/>
        <xsl:value-of select="generate-id()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!-- Providing list of exercises -->
  <xsl:template match="db:book[not(db:part)]|db:part">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()"/>
      <xsl:if test=".//db:qandaset">
        <appendix>
          <title>List of Exercises</title>
          
          <table border="1" xml:id="{@xml:id}tableExercise">
            <caption>
              <xsl:text>Exercises in </xsl:text>
              <emphasis>
                <xsl:value-of select="db:info/db:title" />
              </emphasis>
            </caption>
            <col width="25%" />
            <col width="45%" />
            <col width="30%" />
            <tr>
              <th>Chapter</th>
              <th>Exercise title / page</th>
              <th>Status of completion</th>
            </tr>
            
            <xsl:for-each-group select=".//db:qandaset" group-by="ancestor::db:chapter/@xml:id">
              <xsl:for-each select="current-group()">
                <tr>
                  <xsl:if test="1 = position()"> <!-- Group's key entry only present in first row -->
                    <td valign="top" rowspan="{count(current-group())}">
                      <xref linkend="{current-grouping-key()}"/>
                    </td>
                  </xsl:if>
                  <td>
                    <xsl:apply-templates select="." mode="selectOwnOrInheritedTitle"/>
                  </td>
                  <td/> <!-- empty status of completion -->
                </tr>
              </xsl:for-each>
              
            </xsl:for-each-group>                    
          </table>
          <xsl:apply-templates select="db:chapter[descendant::db:qandaset]" mode="genQuandaset"/>
        </appendix>
      </xsl:if>
    </xsl:copy>        
  </xsl:template>
  
  
  <xsl:template match="db:chapter" mode="genQuandaset">
    <section xml:id="{@xml:id}_qanda">
      <title>
        <xsl:text>Exercises in </xsl:text>
        <emphasis>
          <xsl:for-each select="db:title" >
            <xsl:copy-of select="node()|text()"/>
          </xsl:for-each>
        </emphasis>
      </title>
      <itemizedlist>
        <xsl:for-each select="descendant::db:qandaset">
            <listitem>
              <para>
                <link linkend="{@xml:id}">
                  <xsl:apply-templates select="." mode="selectOwnOrInheritedTitle"/>
                </link>
              </para>
            </listitem>
        </xsl:for-each>                    
      </itemizedlist>  
    </section>
  </xsl:template>
  
  <!-- Excluding non-referenced <bibliography> and <glossary> elements -->
  
  <xsl:template match="db:bibliography/db:biblioentry[not(key('internRef', @xml:id))]">
    <xsl:comment>
      <xsl:text>Excluding non referenced bibliography entry '</xsl:text>
      <xsl:value-of select="db:abbrev"/>
      <xsl:text>', @xml:id='</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>'</xsl:text>
    </xsl:comment>
  </xsl:template>
  
  <xsl:template match="db:glossary//db:glosslist">
    
    <xsl:choose>
      <xsl:when test=".//db:glossentry[key('internRef', @xml:id) or key('internRef', .//*/@xml:id)]">
        <xsl:comment>Non-empty glosslist</xsl:comment>
        <xsl:copy>
          <xsl:apply-templates select="@* | node()"/>
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:comment>
          <xsl:text>No @xml:id or all glosslist entries are not being referenced, excluding whole list.</xsl:text>
        </xsl:comment>
        <xsl:apply-templates select="db:glossentry"/>        
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="db:glossary/db:glosslist//db:glossentry[not(key('internRef', @xml:id)) and not(key('internRef', .//*/@xml:id))]">
    <xsl:comment>
      <xsl:text>Excluding non referenced glossary entry '</xsl:text>
      <xsl:value-of select="db:glossterm"/>
      <xsl:text>', @xml:id='</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>'</xsl:text>
    </xsl:comment>
  </xsl:template>
  
  <!-- 
    <xsl:template match="db:bibliography">
    <db:bibliography>
    <xsl:apply-templates select="*[not(name(.) = 'biblioentry')]|db:biblioentry[key('internRef', @xml:id)]"/>
    </db:bibliography>
    </xsl:template>
    
    
    <xsl:template match="db:glossary/db:glosslist">
    <db:glosslist>
    <xsl:apply-templates select="*[not(name(.) = 'glossentry')]|db:glossentry[key('internRef', @xml:id)]"/>
    </db:glosslist>
    </xsl:template>
  -->
  
  <!-- make annotations -->  
  <xsl:template match="db:annotation[@role='make']">
    <xsl:apply-templates select="*" mode="make"/>
  </xsl:template>
  
  <xsl:template match="db:para[@role='eclipse']" mode='make'>
    
    <xsl:variable name="baseDir">
      <xsl:text>P/</xsl:text>
      <xsl:value-of select="."/>
    </xsl:variable>
    
    <itemizedlist>
      <listitem>
        <para>
          <xsl:text>Maven module source code available at sub directory </xsl:text>
          <filename xlink:href="https://gitlab.mi.hdm-stuttgart.de/goik/GoikLectures/tree/master/{$baseDir}">
            <xsl:value-of select="$baseDir"/>
          </filename>
          <xsl:text> below lecture notes' source code root, see </xsl:text>
          <link linkend="sd1ImportMavenSolutions"> hints regarding import</link>
          <xsl:text>.</xsl:text>
        </para>
      </listitem>
      
      <listitem>
        <para>Online browsing of <link xlink:href="{$baseDir}/target/site/apidocs">API and implementation</link>.</para>
      </listitem>
    </itemizedlist>
  </xsl:template>
  
  <xsl:template match="*" mode="make">
    <db:para>
      <xsl:text>No template of mode='make' defined for element'</xsl:text>
      <xsl:value-of select="name(.)"/>
      <xsl:text>'</xsl:text>
    </db:para>
  </xsl:template>
  
</xsl:stylesheet>
