<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  xmlns="http://docbook.org/ns/docbook" xmlns:db="http://docbook.org/ns/docbook">
  
  <xsl:import href="../../docbook-xsl/xhtml/chunk.xsl"/>
  <xsl:include href="common.xsl"/>
  <xsl:include href="slideReference.xsl"/>
  
  <xsl:output indent="no"/>
  
  <xsl:variable name="topicLookup" select="document('topics.tid.xml', .)"/>
  
  <xsl:template match="@* | node()">
    <xsl:copy>
      <xsl:apply-templates select="node()| @*"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="db:chapter/db:title | db:section/db:title | db:appendix/db:title">
    <xsl:copy-of select="."/>
    <xsl:for-each select="parent::*[contains(@annotations, 'slide') or descendant::*[contains(@annotations, 'slide')]]">
      <para>
        <link xlink:href="__slide{@xml:id}.html">Related slides on offer</link>
      </para>
    </xsl:for-each>
    
  </xsl:template>
  
  <xsl:template match="db:figure/db:title">
    <xsl:variable name="slideContainerId" select="ancestor::*[contains(@annotations, 'slide')]/@xml:id"/>
    
    <title>
      <xsl:copy-of select="text()|*"/>
      
      <xsl:variable name="htmlPresentationForumFileBasename">
        <xsl:text>__figurelink</xsl:text>
        <xsl:value-of select="generate-id()"/>
        <xsl:text>.html</xsl:text>
      </xsl:variable>
      
      <xsl:variable name="htmlPresentationForumFilename">
        <xsl:if test="ancestor::*/@xml:base">
          <xsl:call-template name="hdm.get.leftmost.of.separator">
            <xsl:with-param name="path">
              <xsl:value-of select="ancestor::*/@xml:base"/>
            </xsl:with-param>
            <xsl:with-param name="separator">/</xsl:with-param>
          </xsl:call-template>
          <xsl:text>/</xsl:text>
        </xsl:if>
        <xsl:value-of select="$htmlPresentationForumFileBasename"/>
      </xsl:variable>
      
      <!--
        Hack circumventing docbook XSL bug https://sourceforge.net/p/docbook/bugs/1399
        preventing nesting of <inlinemediaobject> descendants inside <figure>/<title>.
      -->
      
      <xsl:result-document href="target/profile/{$htmlPresentationForumFilename}" method="html" encoding="utf-8" indent="no"
        exclude-result-prefixes="#all" xmlns = "http://www.w3.org/1999/xhtml">
        <span>          
          <xsl:if test="$slideContainerId">                          
            <a>
              <xsl:attribute name="href">
                <xsl:call-template name="slideReference">
                  <xsl:with-param name="slideContainerId" select="$slideContainerId"/>
                  <xsl:with-param name="slideObjectId" select="parent::db:figure/@xml:id"/>
                </xsl:call-template>
              </xsl:attribute>
              <img title="Slide presentation" alt="Slide presentation" style="vertical-align: middle;" 
                src="common/images/presentation.svg" width="40em"/>
            </a>
            <xsl:text> </xsl:text>
          </xsl:if>
          
          <!-- Link to forum -->
          <xsl:call-template name="bbComment">
            <xsl:with-param name="elementId" select="parent::db:figure/@xml:id"/>
          </xsl:call-template>                  
        </span>
      </xsl:result-document>
      
      <xsl:text> </xsl:text>
      
      <xsl:processing-instruction name="dbhtml-include">
        <xsl:text>href="</xsl:text>
        <xsl:value-of select="$htmlPresentationForumFileBasename"/>
        <xsl:text>"</xsl:text>
      </xsl:processing-instruction>
    </title>
    
    <xsl:if test="not($slideContainerId) and ends-with(parent::db:figure/db:mediaobject/db:imageobject/db:imagedata/@fileref, '.fig')">
      <para>
        <uri>
          <xsl:attribute name="xlink:href">
            <xsl:call-template name="hdm.get.leftmost.of.separator">
              <xsl:with-param name="path">
                <xsl:call-template name="relative-uri">
                  <xsl:with-param name="filename" select="parent::db:figure/db:mediaobject/db:imageobject/db:imagedata/@fileref"/>
                </xsl:call-template>
              </xsl:with-param> 
              <xsl:with-param name="separator" select="'.'"/>
            </xsl:call-template>                  
            <xsl:text>.animate.pdf</xsl:text>
          </xsl:attribute>
          <xsl:text>(Animated PDF Version)</xsl:text>
        </uri>
      </para>
    </xsl:if>
    
    <xsl:if test="not($slideContainerId) and ends-with(parent::db:figure/db:mediaobject/db:imageobject/db:imagedata/@fileref, '.svg')">
      <para>
        <uri>
          <xsl:attribute name="xlink:href">
            <xsl:call-template name="relative-uri">
              <xsl:with-param name="filename" select="parent::db:figure/db:mediaobject/db:imageobject/db:imagedata/@fileref"/>
            </xsl:call-template>
          </xsl:attribute>
          <xsl:text>Separate SVG image link</xsl:text>
        </uri>
      </para>
    </xsl:if>   
  </xsl:template>
  
  <xsl:template match="db:qandaset/db:title">
    
    <title>
      <xsl:copy-of select="text()|*"/>
      
      <xsl:variable name="htmlPresentationForumFileBasename">
        <xsl:text>__figurelink</xsl:text>
        <xsl:value-of select="generate-id()"/>
        <xsl:text>.html</xsl:text>
      </xsl:variable>
      
      <xsl:variable name="htmlPresentationForumFilename">
        <xsl:if test="ancestor::*/@xml:base">
          <xsl:call-template name="hdm.get.leftmost.of.separator">
            <xsl:with-param name="path">
              <xsl:value-of select="ancestor::*/@xml:base"/>
            </xsl:with-param>
            <xsl:with-param name="separator">/</xsl:with-param>
          </xsl:call-template>
          <xsl:text>/</xsl:text>
        </xsl:if>
        <xsl:value-of select="$htmlPresentationForumFileBasename"/>
      </xsl:variable>
      
      <!--
        Hack circumventing docbook XSL bug https://sourceforge.net/p/docbook/bugs/1399
        preventing nesting of <inlinemediaobject> descendants inside <figure>/<title>.
      -->
      
      <xsl:message>
        <xsl:text>-----------------------------</xsl:text>
        <xsl:value-of select="$htmlPresentationForumFilename"/>
      </xsl:message>
      
      <xsl:result-document href="target/profile/{$htmlPresentationForumFilename}" method="html" encoding="utf-8" indent="no"
        exclude-result-prefixes="#all" xmlns = "http://www.w3.org/1999/xhtml">
        <span>          
          <!-- Link to forum -->
          <xsl:call-template name="bbComment">
            <xsl:with-param name="elementId" select="parent::db:qandaset/@xml:id"/>
          </xsl:call-template>                  
        </span>
      </xsl:result-document>
      
      <xsl:text> </xsl:text>
      
      <xsl:processing-instruction name="dbhtml-include">
        <xsl:text>href="</xsl:text>
        <xsl:value-of select="$htmlPresentationForumFileBasename"/>
        <xsl:text>"</xsl:text>
      </xsl:processing-instruction>
    </title>
  </xsl:template>  
  
  <xsl:template match="db:imagedata[ends-with(@fileref, '.fig')]">   
    <xsl:copy>
      <xsl:attribute name="fileref">
        <xsl:value-of select="@fileref"/>
        <xsl:text>.svg</xsl:text>
      </xsl:attribute>
      <xsl:apply-templates select="@*[not(name(.) = 'fileref')] | node()"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="db:imagedata[ends-with(@fileref, '.pdf')]">   
    <xsl:copy>
      <xsl:attribute name="fileref">
        <xsl:value-of select="@fileref"/>
        <xsl:text>.svg</xsl:text>
      </xsl:attribute>
      <xsl:apply-templates select="@*[not(name(.) = 'fileref')] | node()"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template name="bbComment">
    <xsl:param name="elementId"/>    
    <xsl:if test="$topicLookup/map/entry[@id = $elementId]">
      <xsl:variable name="tid" select="$topicLookup/map/entry[@id = $elementId]/@tid"/>
      <span class="bbForum" data-tid="$tid">
        <a target="_blank">
          <xsl:attribute name="href">
            <xsl:text>https://bb.mi.hdm-stuttgart.de/topic/</xsl:text>
            <xsl:value-of select="$tid"/>
          </xsl:attribute>
          <img src="common/images/comment.svg" width="40ex" style="vertical-align: middle;"
              alt="Create comment in forum" title="Create comment in forum" />          
        </a>
      </span>
    </xsl:if>    
  </xsl:template>  
  
</xsl:stylesheet>
