<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  xmlns:svg="http://www.w3.org/2000/svg"
  xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"
  xmlns:jessyink="https://launchpad.net/jessyink"
  xmlns:d="http://docbook.org/ns/docbook">
  
  
  <xsl:import href="../../docbook-xsl/xhtml/chunk.xsl"/>
  
  <xsl:include href="common.xsl"/>
  
  <xsl:template match="/">
    
    <xsl:variable name="figFiles" select="//d:imagedata[ends-with(@fileref, '.fig') ]" />
    <xsl:variable name="pdfFiles" select="//d:imagedata[ends-with(@fileref, '.pdf') ]" />
    <xsl:variable name="animatedSvgFiles" select="//d:imagedata[ends-with(@fileref, '.multi.svg') ]" />
    
    <xsl:apply-templates select="//d:imagedata[ends-with(@fileref, '.fig') ]" mode="printLatex"/>
    <xsl:apply-templates select="$animatedSvgFiles" mode="sortLayer"/>
    
    <xsl:result-document method="text" encoding="utf-8" href="mediaobjects.list" >
      <xsl:text>all: </xsl:text>
          
      <xsl:for-each select="$figFiles">
        <xsl:call-template name="relative-uri">
          <xsl:with-param name="destdir">
            <xsl:call-template name="dbhtml-dir">
              <xsl:with-param name="context" select="."/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:call-template>
        
        <xsl:value-of select="@fileref"/>
        <xsl:text>.pdf </xsl:text>
      </xsl:for-each>
      
      <xsl:for-each select="$pdfFiles">
        <xsl:call-template name="relative-uri">
          <xsl:with-param name="destdir">
            <xsl:call-template name="dbhtml-dir">
              <xsl:with-param name="context" select="."/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:call-template>
        
        <xsl:value-of select="@fileref"/>
        <xsl:text>.svg </xsl:text>
      </xsl:for-each>
      
      <xsl:for-each select="//d:videodata[ends-with(@fileref, '.webm') ]">
        <xsl:call-template name="relative-uri">
          <xsl:with-param name="destdir">
            <xsl:call-template name="dbhtml-dir">
              <xsl:with-param name="context" select="."/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:call-template>
        <xsl:value-of select="@fileref"/>
        <xsl:text>.mp4 </xsl:text>
      </xsl:for-each>
      
    </xsl:result-document>
  </xsl:template>  
  
  <xsl:template match="d:imagedata" mode="sortLayer">
    <xsl:variable name="svgMulti" select="document(@fileref, .)"/>
    
    <xsl:value-of select="@fileref"/>
    <xsl:text>.files </xsl:text>
    
    
    <xsl:apply-templates select="$svgMulti/svg:svg" mode="sortLayer">
      <xsl:with-param name="svgMultiBasename">
        <xsl:call-template name="hdm.get.leftmost.of.separator">
          <xsl:with-param name="path">
            
              <xsl:if test="ancestor::*/@xml:base">
                <xsl:call-template name="hdm.get.leftmost.of.separator">
                  <xsl:with-param name="path" select="ancestor::*/@xml:base"/>
                  <xsl:with-param name="separator" select="'/'"/>
                </xsl:call-template>
                <xsl:text>/</xsl:text>
              </xsl:if>
            <xsl:value-of select="@fileref"/>
            
          </xsl:with-param>
          <xsl:with-param name="separator" select="'.multi.svg'"/>
        </xsl:call-template>
      </xsl:with-param>
      <xsl:with-param name="svgRoot" select="$svgMulti/svg:svg"/>
    </xsl:apply-templates>
    
  </xsl:template>
  
  <xsl:template match="svg:svg" mode="sortLayer">
    <xsl:param name="svgMultiBasename"/>
    <xsl:param name="svgRoot"/>
    <xsl:variable name="masterSlide" select="svg:g[@jessyink:masterSlide]"/>
    
    <xsl:variable name="displayLayers" select="svg:g[not(@jessyink:masterSlide)]"/>
        
    <xsl:result-document href="{$svgMultiBasename}.filenames" exclude-result-prefixes="#all" indent="yes">
      <start>
        <xsl:for-each select="$displayLayers">
          <xsl:sort select="position()" data-type="number" order="descending" />
          <!-- 
          <xsl:sort select="@inkscape:label" />
          -->
          <xsl:variable name="svgLayerBasefilename">
            <xsl:value-of select="$svgMultiBasename"/>
            <xsl:text>-</xsl:text>
            <xsl:value-of select="position() - 1"/>
          </xsl:variable>
          <file>
            <xsl:value-of select="$svgLayerBasefilename"/>
          </file>
          <xsl:result-document href="{$svgLayerBasefilename}.svg" exclude-result-prefixes="svg">
            <svg xmlns="http://www.w3.org/2000/svg">
              <xsl:copy-of select="$svgRoot/@*" />
              <xsl:copy-of select="$svgRoot/*[not (name(.) = 'g')]"/>
              <xsl:apply-templates select="." mode="sortLayer"/>
              <xsl:copy-of select="$masterSlide"/>
            </svg>
          </xsl:result-document>
        </xsl:for-each>
      </start>
    </xsl:result-document>
  </xsl:template>
  
  <xsl:template match="svg:g" mode="sortLayer">
    <svg:g>
      <xsl:copy-of select="@*[not(name(.)='style')]|*"/>      
    </svg:g>
  </xsl:template>

  <xsl:template match="*" mode="sortLayer">
    <xsl:message>
      <xsl:text>Error: No template defined for element '</xsl:text>
      <xsl:value-of select="name(.)"/>
      <xsl:text>' in mode sortLayer</xsl:text>
    </xsl:message>
  </xsl:template>
  
  
  <xsl:template match="d:imagedata" mode="printLatex">
    
    <xsl:variable name="dirPath">
      <xsl:call-template name="hdm.get.leftmost.of.separator">
        <xsl:with-param name="path" select="@fileref"/>
        <xsl:with-param name="separator" select="'/'"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="fileName">
      <xsl:call-template name="hdm.get.pure.filename">
        <xsl:with-param name="path" select="@fileref"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="fileBaseName">
      <xsl:call-template name="hdm.get.leftmost.of.separator">
        <xsl:with-param name="path" select="$fileName"/>
        <xsl:with-param name="separator" select="'.'"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="fullGenerateBasename" >
      <xsl:call-template name="relative-uri">
        <xsl:with-param name="destdir">
          <xsl:call-template name="dbhtml-dir">
            <xsl:with-param name="context" select="."/>
          </xsl:call-template>
        </xsl:with-param>
      </xsl:call-template>
      <xsl:value-of select="$dirPath"/>
      <xsl:text>/Generate/</xsl:text>
      <xsl:value-of select="$fileBaseName"/>
    </xsl:variable>
    
    
    <xsl:value-of select="$fullGenerateBasename"/>
    <xsl:text>.tex
    </xsl:text>
    
    <xsl:result-document method="text" encoding="utf-8" href="{$fullGenerateBasename}.tex" >
      <xsl:text>\input{/usr/share/hdm-docbook-xsl/Preprocess/pre.tex} 
        \begin{mycenterframe}{</xsl:text>
      <xsl:apply-templates select="." mode="object.title.markup.textonly"/>
      <xsl:text>}
        \multifigure{Generate/</xsl:text>
      <xsl:value-of select="$fileBaseName" />
      <xsl:text>}
        \end{mycenterframe}
        \end{document}
      </xsl:text>
    </xsl:result-document>
  </xsl:template>
  
  <xsl:template name="hdm.get.pure.filename">
    <!-- Stripping a filename path from all directory components -->
    <xsl:param name="path"/>
    <xsl:choose>
      <xsl:when test="contains($path, '/' )">
        <xsl:call-template name="hdm.get.pure.filename">
          <xsl:with-param name="path">
            <xsl:value-of select="substring-after($path, '/')"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$path"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
</xsl:stylesheet>
