package dom.xsl;
import java.sql.*;
/** Database access layer
 *
 */
public class DbAccess {

  /** Open a connection to a RDBMS.
   * @param jdbcUrl The database server's address
   * @param userName The user's name
   * @param password The user's password
   */
  public void connect(final String jdbcUrl, 
      final String userName, final String password) {
    try {
      conn = DriverManager.getConnection(jdbcUrl, userName, password);
      priceQuery = conn.prepareStatement(sqlPriceQuery);
    } catch (SQLException e) {
      System.err.println("Unable to open connection to database:" + e);}
  }
  /** Read the price from the RDBMS for a given article number. In a "real"
   *  application we'd use an interger type rather than a string.
   * @param articleNumber
   *   The numberb of the article for which we search pricing information.
   * @return
   *   The price if available, else an error message.
   */
  public String readPrice(final String articleNumber) {
    String result;
    try {
      priceQuery.setString(1, articleNumber);
      final ResultSet rs = priceQuery.executeQuery();
      if (rs.next()) {
        result = rs.getString("price");
      } else {
        result = "No price available for article '" + articleNumber + "'";
      }
    } catch (SQLException e) {
      result = "Error reading price for article '" + articleNumber + "':" + e;
    }
    return result;
  }
  /**
   * Close the database connection.
   */
  public void close() {
    try {conn.close();} catch (SQLException e) {
      System.err.println("Error closing database connection:" + e);
    }
  }
  private static final String sqlPriceQuery = 
      "SELECT price FROM Product WHERE orderNo = ?";
  private PreparedStatement priceQuery = null;
  private Connection conn = null;
}
