package dom.xsl;

import java.io.IOException;

import org.jdom2.Document;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;
import org.jdom2.transform.XSLTransformException;
import org.jdom2.transform.XSLTransformer;


import dom.MySaxErrorHandler;

/**
 * Transforming an XML document to HTML by using
 * an XSL stylesheet.
 * @author goik
 *
 */
public class Xml2Html {
   private final SAXBuilder builder = new SAXBuilder();

   final XSLTransformer transformer;

   /**
    * 
    * @param xslFilename the stylesheet being used for subsequent
    * transformations by {@link #transform(String, String)}.
    * 
    * @throws XSLTransformException XSL script may be ill-defined
    */
   public Xml2Html(final String xslFilename) throws XSLTransformException {
      builder.setErrorHandler(new MySaxErrorHandler(System.err));
      transformer =  new XSLTransformer(xslFilename);
   }
   /**
    * The actual workhorse
    * 
    * @param xmlInFilename input file to be transformed
    * @param resultFilename the result file holding the generated HTML document
    * @throws JDOMException The transformation may fail for various reasons.
    * @throws IOException File access problem or corrupt content.
    */
   public void transform(final String xmlInFilename,
         final String resultFilename) throws JDOMException, IOException {

      final Document inDoc = builder.build(xmlInFilename);
      Document result = transformer.transform(inDoc);

      // Set formatting for the XML output
      final Format outFormat = Format.getPrettyFormat();

      // Serialize to console
      final XMLOutputter printer = new XMLOutputter(outFormat);
      printer.output(result.getDocument(), System.out);

   }
}
