package sax.view;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/** A view on memo documents restricting to sender name an subject. */
public class MemoViewHandler extends DefaultHandler {

  // These variables help us to keep track of the current event state spanning
  // each startElement(...) -- character(...) -- endElement(...) event sequence 
  boolean inFromContext = false,
      inSubjectContext = false;
  
  public void startElement(String namespaceUri, String localName,
      String rawName, Attributes attrs) {
    switch(rawName) {
    case "from":
      inFromContext = true;
      System.out.print("Sender: ");
      break;
    case "subject": 
      inSubjectContext = true;
      System.out.print("Subject: ");
      break;
    case "surname":
      if (inFromContext) {
        System.out.print(" "); // Adding additional space between <name> and <surname>  content.
      }
      break;
    }
  }

  @Override
  public void endElement(String uri, String localName, String rawName)
      throws SAXException {
    switch(rawName) {
    case "from": 
      inFromContext = false;
      System.out.println();
      break;
    case "subject": 
      inSubjectContext = false;
      System.out.println();
      break;
    }
  }

  @Override
  public void characters(char[] ch, int start, int length) throws SAXException {
    if (inFromContext || inSubjectContext) {
      System.out.print(new String(ch, start, length));
    }
  }
}