package sda.jdbc.intro.auth;

import java.util.regex.Pattern;

import javax.swing.JComponent;
import javax.swing.JTextField;

/**
 * We may check user input from text fields in a generic way. 
 * defined by regular expressions. In addition error messages are forwarded
 * automatically to respective GUI elements.
 * 
 * @author goik
 */
public class RegexpVerifier extends InputVerifierNotify {

   final Pattern syntaxPattern;
   
   /**
    * For each input field to be validated we need the
    * following parameters:
    * 
    * @param field The input field to be validated.
    * 
    * @param regex The regular expression defining allowed textual content. The string
    *                "^[^ ]*$" for example allows arbitrary text not containing any spaces.
    *                
    * @param errMsg      This text will be presented to the user in case of a validation error.
    */
   public RegexpVerifier (final JTextField field, final String regex, final String errMsg) {
      super(field, errMsg);
      syntaxPattern = Pattern.compile(regex);
   }

   @Override
   public boolean verify(JComponent input) {
      if (input instanceof JTextField) {
         final String userInput =  ((JTextField) input).getText();
         if (syntaxPattern.matcher(userInput).find()) {
            validationLabel.setText("");
            return true;
         } else {
            validationLabel.setText(errorMessage);
         }
      }
      return false;
   }
}