package sda.jdbc.intro.v1.sanitize;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.SQLException;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import sda.jdbc.intro.v1.PersistenceHandler;

public class InsertPerson extends JFrame {
   
   private static final long serialVersionUID = 6815975741605247675L;
   
   final PersistenceHandler persistenceHandler = new PersistenceHandler();
   
   final JTextField nameField = new JTextField(15);
   final JLabel    nameFieldValidationLabel = new JLabel();
   final RegexpVerifier nameFieldVerifier = new RegexpVerifier(
                           "^[^;'\"]+$",
                           "No special characters");
   
   final JTextField emailField = new JTextField(20);
   final JLabel    emailFieldValidationLabel = new JLabel();
   final RegexpVerifier emailFieldVerifier =
         new RegexpVerifier("^[\\w\\-\\.\\_]+@[\\w\\-\\.]*[a-zA-Z]{2,4}$",
                           "email not valid");

   final JButton toggleConnectButton = new JButton(),
                        insertButton = new JButton("Insert");

   final JPanel databaseFieldPanel = new JPanel();
   JLabel validationLabel;
   

   private void setDatabaseConnectionState(final boolean state) {
      if (state) {
         toggleConnectButton.setText("Disconnect");
      } else {
         toggleConnectButton.setText("Connect");
      }
      for (final Component c: databaseFieldPanel.getComponents()){
         c.setEnabled(state);
      }
   }

   public static void main(String[] args) throws SQLException {
      InsertPerson app = new InsertPerson();
      app.setVisible(true);
   }
   
   public InsertPerson (){
      super ("Add a person's data");
      
      setSize(500, 500);
      
      addWindowListener(new WindowAdapter() {
         // In case a user closes our Jframe window while still being connected
         // we have to close the database connection.
         @Override
         public void windowClosing(WindowEvent e) {
            super.windowClosing(e);
            if (persistenceHandler.isConnected() && !persistenceHandler.disconnect()) {
               System.exit(1);
            } else {
               System.exit(0);
            }
         }
      });
      Box top = Box.createHorizontalBox();
      add(top, BorderLayout.NORTH);
      top.add(toggleConnectButton);
      
      toggleConnectButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed(ActionEvent e) {
            if (persistenceHandler.isConnected()) {
               if (persistenceHandler.disconnect()){
                  setDatabaseConnectionState(false);
               } else {
                  JOptionPane.showMessageDialog(null, persistenceHandler.getErrorMessage());
               }
            } else {
               if (persistenceHandler.connect()){
                  setDatabaseConnectionState(true);
               } else {
                  JOptionPane.showMessageDialog(null, persistenceHandler.getErrorMessage());
               }
            }
         }
      });
      
      databaseFieldPanel.setLayout(new GridLayout(0, 3)); //Third column for validation label
      add(databaseFieldPanel);

      databaseFieldPanel.add(new JLabel("Name:"));
      databaseFieldPanel.add(nameField);
      nameFieldValidationLabel.setForeground(Color.RED);
      databaseFieldPanel.add(nameFieldValidationLabel);
      nameField.setInputVerifier(nameFieldVerifier);
      
      databaseFieldPanel.add(new JLabel("E-mail:"));
      databaseFieldPanel.add(emailField);
      databaseFieldPanel.add(emailFieldValidationLabel);
      emailFieldValidationLabel.setForeground(Color.RED);
      emailField.setInputVerifier(emailFieldVerifier);
      
      insertButton.addActionListener(new ActionListener() {
         @Override
         public void actionPerformed(ActionEvent e) {
            if (!nameFieldVerifier.inputIsValid() || !emailFieldVerifier.inputIsValid()) {
               JOptionPane.showMessageDialog(null, "Invalid input value(s)");
            } else {
               if (persistenceHandler.add(nameField.getText(), emailField.getText())) {
                  nameField.setText("");
                  emailField.setText("");
                  JOptionPane.showMessageDialog(null, "Succesfully inserted dataset");
               } else {
                  JOptionPane.showMessageDialog(null, persistenceHandler.getErrorMessage());
               }
            }
         }
      });
      databaseFieldPanel.add(Box.createGlue());
      databaseFieldPanel.add(insertButton);   
      setDatabaseConnectionState(false);
      pack();
   }
}
