package party1;


/**
 * @author goik
 * 
 * Provide statistical information regarding a party. We distinguish
 * between the numbers of guests and the number of organizers.
 * The whole party will cost a certain amount of money being set 
 * by the {@link #setExpenses(float)} method.
 * 
 * These expenses will be paid by the organizers the latter being set
 * in {@link #setNumberOfOrganizers(int)}. Each organizer will have to
 * pay his corresponding share.
 *
 */
public interface PartyStatistics {
    
    /**
     * @param numGuests The number of guests attending the party,
     *                default value is zero.
     */
    public void setNumberOfGuests(int numGuests);
    
    /**
     * @param numOrganizers The number of persons organizing the party,
     *   default value is zero.
     */
    public void setNumberOfOrganizers(int numOrganizers);
    
    /**
     * 
     * @param expenses The sum of all costs the party has caused.
     *                Default value is zero (0f)
     */
    public void setExpenses(float expenses);
    
    /**
     * We assume that all party participants shake hands with each
     * other at arrival. Example: Having one organizer and three guests
     * results in six handshakes.
     * 
     * @return The total number of handshakes among all guests and organizers.
     */
    public int getNumberOfHandshakes();
    
    /**
     * Expenses will be shared by all organizers.
     * 
     * Precondition: The number of organizers (see {@link #setNumberOfOrganizers(int)}
     *             must be positive.
     * 
     * @return Each organizer's share according to {@link #setExpenses(float)} and 
     *           {@link #setNumberOfOrganizers(int)}
     *
     */
    public float getAverageExpenses();
    

}
