package xml2rdbms.solution;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Extending our billing handler reading relational data.
 *
 */
public class BillingHandler extends DefaultHandler {
	
	String currentOpenElement = null;
	
	int transactionId, quantity; 
	String partNum, comment;
	
	final DatabaseAccess dba = new DatabaseAccess();

	@Override
	public void startDocument() throws SAXException {
		dba.connect();
	}

	@Override
	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		currentOpenElement = qName; // Required state information in subsequent calls to
									// characters(char[] ch, int start, int length)

		if (currentOpenElement.equals("item")) { // Starting new record
			transactionId = Integer.parseInt(attributes.getValue("transactionId"));
			partNum = attributes.getValue("partNum");
			comment = null; // Reset in case element is absent in current record
		}
	}

	@Override
	public void characters(char[] ch, int start, int length)
			throws SAXException {
		if (null != currentOpenElement ) {
			final String content = new String(ch, start, length);
			switch(currentOpenElement) {
			case "quantity": quantity = Integer.parseInt(content); break;
			case "comment": comment = content; break;
			}
		}
 	}

	@Override
	public void endDocument() throws SAXException {
		dba.close();
	}

	@Override
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		if (qName.equals("item")) { // Write current record to database
			dba.insert(transactionId, partNum, quantity, comment);
		}
		currentOpenElement = null;
	}
}
