package hdm.utils.xmlparse;

import gnu.getopt.Getopt;

import java.io.IOException;

import org.apache.xerces.parsers.DOMParser;
import org.xml.sax.SAXException;

public class Parser {
  public static void main(String [] argv){
     
    final StringBuffer sb = new StringBuffer();

    LongoptExplanation.add(false, "help",     sb, OPTION_HELP);
    LongoptExplanation.add(false, "version",  sb, OPTION_VERSION);
    LongoptExplanation.add(false, "validate", sb, OPTION_VALIDATION);
    LongoptExplanation.add(false, "schema",   sb, OPTION_SCHEMAVALIDATION);
    LongoptExplanation.add(false, "debug",    sb, OPTION_DEBUG);

    if (0 == argv.length){
      printUsage();
      System.exit(0);
    }
    Getopt g = new Getopt
      ("domParser", 
       argv, 
       "-" + LongoptExplanation.getShortOptString(), 
       LongoptExplanation.getOptions()
       );
    int c;
    while ((c = g.getopt()) != -1){
      switch(c){
      case 0:
        final int longOptionCode = (new Integer(sb.toString())).intValue();
        processOptionCode(longOptionCode, g);
      case 1://
        if (null == xmlFileName){//Not yet initialized?
          xmlFileName = g.getOptarg();
        } else {//Only one file to parse may be specified!
          printUsage();
          System.exit(1);
        }
        break;
      case '?':
        break; // getopt() already printed an error
      default:
        processOptionCode(c, g);
      }
    }
    if (null == xmlFileName){
      System.err.println("No File for parsing specified");
      System.exit(1);
    }
    final ParserHandler parserHandler = new ParserHandler();
    try {
      DOMParser parser = new DOMParser();
      parser.setErrorHandler(parserHandler);
      
      if (doValidation) {
         parser.setFeature("http://xml.org/sax/features/validation", doValidation);
      }
      
      if (doSchemaValidation) {
         parser.setFeature("http://apache.org/xml/features/xinclude", true);
         parser.setFeature("http://apache.org/xml/features/validation/schema-full-checking", true);
         parser.setFeature("http://apache.org/xml/features/validation/schema", doSchemaValidation);
      }
      parser.setEntityResolver(parserHandler);
      parser.parse(xmlFileName);
      if (doDebugging){
        System.out.println(parserHandler.getLogMessages());
      }
      System.out.println("Parsing was successfull");
    } 
    catch (SAXException e) {
      System.err.println("parsing error");
      System.exit(1);
    } 
    catch (IOException e) {
      System.err.println("Error on opening file, processed entity references:");
      System.err.println(parserHandler.getLogMessages());
      System.err.println(e);
      System.exit(1);
    }
  }
  private static void processOptionCode(int optionCode, Getopt getopt){
    switch(optionCode){
    case OPTION_HELP:
      printUsage();
      System.exit(0);
      break;

    case OPTION_VALIDATION:
      doValidation = true;
      break;

    case OPTION_SCHEMAVALIDATION:
      doValidation = true;
      doSchemaValidation=true;
      break;

    case OPTION_DEBUG:
      doDebugging = true;
      break;

    case OPTION_VERSION:
      System.out.print("Version " + version + "\n");
      System.exit(0);
      break;

    default:
      System.err.println
        ("processOptionCode:No such long option code:" +
         optionCode  + ":" + (char)optionCode);
      System.exit(1);
    }
  }
  private  static void printUsage(){
    System.err.println
      ("Usage: xmlparse [options] filename" + "\n\noptions:" +
       LongoptExplanation.getUsage());
  }
  private static String xmlFileName = null;

  private static final int OPTION_HELP = 'h';
  private static final int OPTION_VERSION = 'v';
  private static final int OPTION_VALIDATION = 'V';
  private static final int OPTION_SCHEMAVALIDATION = 's';
  private static final int OPTION_DEBUG = 'd';

  private static boolean doValidation = false;
  private static boolean doSchemaValidation = false;
  private static boolean doDebugging = false;

  private static final String version="0.9";
}
